##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for Document module.
"""

import unittest
import Testing

from os.path import abspath
from os.path import dirname
from os.path import join as path_join
from re import compile
from StringIO import StringIO

from DocumentTemplate.DT_Util import html_quote
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from Products.PythonScripts.PythonScript import PythonScript
from zope.component import getSiteManager
from zope.interface import implements
from zope.interface.verify import verifyClass

from Products.CMFCore.interfaces import ILinebreakNormalizer
from Products.CMFCore.testing import ConformsToContent
from Products.CMFCore.tests.base.content import BASIC_HTML
from Products.CMFCore.tests.base.content import BASIC_ReST
from Products.CMFCore.tests.base.content import BASIC_STRUCTUREDTEXT
from Products.CMFCore.tests.base.content import DOCTYPE
from Products.CMFCore.tests.base.content import ENTITY_IN_TITLE
from Products.CMFCore.tests.base.content import FAUX_HTML_LEADING_TEXT
from Products.CMFCore.tests.base.content import HTML_TEMPLATE
from Products.CMFCore.tests.base.content import ReST_NO_HEADERS
from Products.CMFCore.tests.base.content import ReST_NO_HEADERS_BUT_COLON
from Products.CMFCore.tests.base.content import ReST_WITH_HTML
from Products.CMFCore.tests.base.content import SIMPLE_STRUCTUREDTEXT
from Products.CMFCore.tests.base.content import SIMPLE_XHTML
from Products.CMFCore.tests.base.content import STX_NO_HEADERS
from Products.CMFCore.tests.base.content import STX_NO_HEADERS_BUT_COLON
from Products.CMFCore.tests.base.content import STX_WITH_HTML
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.testcase import TransactionalTest
from Products.CMFCore.tests.base.tidata import FTIDATA_CMF
from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
from Products.CMFCore.TypesTool import TypesTool
from Products.CMFDefault import utils


class TransactionalTestBase(TransactionalTest):

    def _getTargetClass(self):
        from Products.CMFDefault.Document import Document

        return Document

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)


class DummyLinebreakNormalizer(object):

    implements(ILinebreakNormalizer)

    def __init__(self, before, after):
        self.before = before
        self.after = after

    def normalizeIncoming(self, obj, text):
        return text.replace(self.before, self.after)

    def normalizeOutgoing(self, ob, text):
        return text.replace(self.before, self.after)


class DocumentTests(ConformsToContent, TransactionalTestBase):

    def test_interfaces(self):
        from Products.CMFDefault.interfaces import IDocument
        from Products.CMFDefault.interfaces import IMutableDocument

        verifyClass(IDocument, self._getTargetClass())
        verifyClass(IMutableDocument, self._getTargetClass())

    def test_Empty(self):
        d = self._makeOne('foo', text_format='structured-text')

        self.assertEqual(d.title, '')
        self.assertEqual(d.description, '')
        self.assertEqual(d.text, '')
        self.assertEqual(d.text_format, 'structured-text')
        self.assertEqual(d._stx_level, 1)
        self.assertEqual(d.get_size(), 0)

    def test_editBasicHTML(self):
        d = self._makeOne('foo')
        d.edit('html', BASIC_HTML)

        self.assertTrue(hasattr(d, 'cooked_text'))
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.text.find('</body>'), -1)
        self.assertEqual(d.cooked_text, '\n  <h1>Not a lot here</h1>\n ')

        # Since the format is html, the STX level operands should
        # have no effect.
        d.CookedBody(stx_level=3, setlevel=1)
        self.assertEqual(d._stx_level, 1)

    def test_editSimpleXHTML(self):
        d = self._makeOne('foo')
        d.edit('html', SIMPLE_XHTML)

        self.assertTrue(hasattr(d, 'cooked_text'))
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.cooked_text, '\n  <h1>Not a lot here</h1>\n ')

    def test_UpperedHtml(self):
        self.REQUEST['BODY'] = BASIC_HTML.upper()
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.title, 'TITLE IN TAG')
        self.assertEqual(d.text.find('</BODY'), -1)
        self.assertEqual(d.Description(), 'DESCRIBE ME')
        self.assertEqual(len(d.Contributors()), 3)

    def test_EntityInTitle(self):
        self.REQUEST['BODY'] = ENTITY_IN_TITLE
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.title, '&Auuml;rger')

    def test_HtmlWithDoctype(self):
        self.REQUEST['BODY'] = '%s\n%s' % (DOCTYPE, BASIC_HTML)
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Description(), 'Describe me')

    def test_HtmlWithoutNewlines(self):
        self.REQUEST['BODY'] = ''.join((BASIC_HTML.split('\n')))
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Description(), 'Describe me')

    def test_EditPlainDocumentWithEmbeddedHTML(self):
        d = self._makeOne('foo')
        d.edit('structured-text', FAUX_HTML_LEADING_TEXT)
        fully_edited = d.cooked_text
        d._edit(FAUX_HTML_LEADING_TEXT)
        partly_edited = d.cooked_text
        self.assertEqual(fully_edited, partly_edited)

    def test_BigHtml(self):
        d = self._makeOne('foo')
        s = []
        looper = '<li> number %s</li>'
        for i in range(12000): s.append(looper % i)
        body = '<ul>\n%s\n</ul>' % '\n'.join(s)
        self.REQUEST['BODY'] = HTML_TEMPLATE % {'title': 'big document',
                                'body': body}
        d.PUT(self.REQUEST, self.RESPONSE)
        self.assertEqual(d.CookedBody(), body)

    def test_BigHtml_via_upload(self):
        d = self._makeOne('foo')
        s = []
        looper = '<li> number %s</li>'
        for i in range(12000): s.append(looper % i)
        body = '<ul>\n%s\n</ul>' % '\n'.join(s)
        html = HTML_TEMPLATE % {'title': 'big document',
                                'body': body}
        _file = StringIO(html)
        d.edit(text_format='html', text='', file=_file)
        self.assertEqual(d.CookedBody(), body)

    def test_Htmltag_removal_and_formatchange(self):
        # Test for http://www.zope.org/Collectors/CMF/214
        d = self._makeOne('foo')
        quoted_html = html_quote(BASIC_HTML)

        # Put HTML into a plain text document
        d.edit(text_format='plain', text=BASIC_HTML)
        new_body = d.CookedBody()
        self.assertFalse(new_body == BASIC_HTML)
        self.assertTrue(new_body.startswith(quoted_html[:5]))

        # Now we change the format. The body *must* change because
        # the format change will trigger re-cooking
        old_body = d.CookedBody()
        d.setFormat('html')
        new_body = d.CookedBody()
        self.assertFalse(old_body == new_body)
        self.assertFalse(new_body == BASIC_HTML)

    def test_Html_Fragment(self):
        # Test that edits with HTML fragments behave nicely
        FRAGMENT = '<div id="placeholder">CONTENT</div>'
        d = self._makeOne('foo')
        d.edit(text_format='html', text=FRAGMENT)
        self.assertEqual(d.CookedBody(), FRAGMENT)
        self.assertEqual(d.get_size(), len(FRAGMENT))

    def test_plain_text(self):
        # test that plain text forrmat works
        PLAIN_TEXT = '*some plain text*\nwith a newline'
        d = self._makeOne('foo')
        d.edit(text_format='plain', text=PLAIN_TEXT)
        self.assertEqual(d.CookedBody(),
                          '*some plain text*<br />with a newline')
        self.assertEqual(d.get_size(), len(PLAIN_TEXT))

    def test_EditStructuredTextWithHTML(self):
        d = self._makeOne('foo')
        d.edit(text_format='structured-text', text=STX_WITH_HTML)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.get_size(), len(STX_WITH_HTML))

    def test_StructuredText(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertTrue(hasattr(d, 'cooked_text'))
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.Title(), 'My Document')
        self.assertEqual(d.Description(), 'A document by me')
        self.assertEqual(len(d.Contributors()), 3)
        self.assertTrue(d.cooked_text.find('<p>') >= 0)
        self.assertTrue(d.CookedBody().find('<h1') >= 0)

        # Make sure extra HTML is NOT found
        self.assertTrue(d.cooked_text.find('<title>') < 0)
        self.assertTrue(d.cooked_text.find('<body>') < 0)

        # test subject/keyword headers
        subj = list(d.Subject())
        self.assertEqual(len(subj), 4)
        subj.sort()
        self.assertEqual(subj, ['content management', 'framework',
                                'unit tests', 'zope'])

    def test_STX_Levels(self):
        d = self._makeOne('foo')
        d.edit(text_format='structured-text', text=BASIC_STRUCTUREDTEXT)
        self.assertEqual(d._stx_level, 1)

        ct = d.CookedBody()
        self.assertTrue(ct.find('<h1') >= 0)
        self.assertEqual(d._stx_level, 1)

        ct = d.CookedBody(stx_level=2)
        self.assertFalse(ct.find('<h1') >= 0)
        self.assertTrue(ct.find('<h2') >= 0)
        self.assertEqual(d._stx_level, 1)

        ct = d.CookedBody(stx_level=2, setlevel=1)
        self.assertFalse(ct.find('<h1') >= 0)
        self.assertTrue(ct.find('<h2') >= 0)
        self.assertEqual(d._stx_level, 2)

        ct = d.CookedBody()
        self.assertEqual(d._stx_level, 2)
        self.assertFalse(d.CookedBody().find('<h1') >= 0)
        self.assertTrue(d.CookedBody().find('<h2') >= 0)

    def test_ReStructuredText(self):
        from Products.CMFDefault.Document import REST_AVAILABLE
        if not REST_AVAILABLE:
            return
        self.REQUEST['BODY'] = BASIC_ReST
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)
        d.edit(text_format='restructured-text', text=BASIC_ReST)

        self.assertTrue(hasattr(d, 'cooked_text'))
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.Title(), 'My Document')
        self.assertEqual(d.Description(), 'A document by me')
        self.assertEqual(len(d.Contributors()), 3)

        # CookedBody() for consistency
        self.assertTrue(d.CookedBody().find('<p>') >= 0)

        #  override default
        self.assertTrue(d.CookedBody(rest_level=0).find('<h1') >= 0)

        # we should check for the list
        self.assertTrue(d.CookedBody().find('<ul') >= 0)

        # Make sure extra HTML is NOT found
        self.assertTrue(d.cooked_text.find('<title>') < 0)
        self.assertTrue(d.cooked_text.find('<body>') < 0)

        # test subject/keyword headers
        subj = list(d.Subject())
        self.assertEqual(len(subj), 4)
        subj.sort()
        self.assertEqual(subj, ['content management', 'framework',
                                'unit tests', 'zope'])

    def test_EditReStructuredTextWithHTML(self):
        from Products.CMFDefault.Document import REST_AVAILABLE
        if not REST_AVAILABLE:
            return
        d = self._makeOne('foo')
        d.edit(text_format='restructured-text', text=ReST_WITH_HTML)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.get_size(), len(ReST_WITH_HTML))

    def test_ReST_Levels(self):
        from Products.CMFDefault.Document import REST_AVAILABLE
        if not REST_AVAILABLE:
            return
        d = self._makeOne('foo')
        d.edit(text_format='restructured-text', text=BASIC_ReST)
        d.CookedBody(rest_level=0, setlevel=True)
        self.assertEqual(d._rest_level, 0)

        ct = d.CookedBody()

        # ReST always augments the level by one
        self.assertTrue(ct.find('<h1') >= 0)
        self.assertEqual(d._rest_level, 0)

        ct = d.CookedBody(rest_level=1)
        self.assertFalse(ct.find('<h1') >= 0)
        self.assertTrue(ct.find('<h2') >= 0)
        self.assertEqual(d._rest_level, 0)

        ct = d.CookedBody(rest_level=1, setlevel=1)
        self.assertFalse(ct.find('<h1') >= 0)
        self.assertTrue(ct.find('<h2') >= 0)
        self.assertEqual(d._rest_level, 1)

        ct = d.CookedBody()
        self.assertEqual(d._rest_level, 1)
        self.assertFalse(d.CookedBody().find('<h1') >= 0)
        self.assertTrue(d.CookedBody().find('<h2') >= 0)

    def test_Init(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.Title(), 'My Document')
        self.assertEqual(d.Description(), 'A document by me')
        self.assertEqual(len(d.Contributors()), 3)
        self.assertTrue(d.cooked_text.find('<p>') >= 0)

        d = self._makeOne('foo', text='')
        self.REQUEST['BODY'] = BASIC_HTML
        d.PUT(self.REQUEST, self.RESPONSE)
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Title(), 'Title in tag')
        self.assertEqual(len(d.Contributors()), 3)

        d = self._makeOne('foo', text_format='structured-text',
                          title='Foodoc')
        self.assertEqual(d.text, '')
        self.assertFalse(d.CookedBody())
        self.assertEqual(d.title, 'Foodoc')
        self.assertEqual(d.Format(), 'text/plain')

        # Tracker issue 435:  initial text is not cooked.
        d = self._makeOne('foo', text_format='structured-text',
                          text=STX_NO_HEADERS)
        self.assertEqual(d.EditableBody(), STX_NO_HEADERS)
        self.assertTrue(d.CookedBody())
        self.assertEqual(d.Format(), 'text/plain')

    def test_STX_NoHeaders(self):
        self.REQUEST['BODY'] = STX_NO_HEADERS
        d = self._makeOne('foo')
        d.editMetadata(title="Plain STX",
                       description="Look, Ma, no headers!",
                       subject=("plain", "STX"))
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Title(), 'Plain STX')
        self.assertEqual(d.Description(), 'Look, Ma, no headers!')
        self.assertEqual(len(d.Subject()), 2)
        self.assertTrue('plain' in d.Subject())
        self.assertTrue('STX' in d.Subject())

        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.Title(), 'Plain STX')
        self.assertEqual(d.Description(), 'Look, Ma, no headers!')
        self.assertEqual(len(d.Subject()), 2)
        self.assertTrue('plain' in d.Subject())
        self.assertTrue('STX' in d.Subject())

    def test_ReST_NoHeaders(self):
        from Products.CMFDefault.Document import REST_AVAILABLE
        if not REST_AVAILABLE:
            return
        self.REQUEST['BODY'] = ReST_NO_HEADERS
        d = self._makeOne('foo')
        d.editMetadata(title="Plain ReST",
                       description="Look, Ma, no headers!",
                       subject=("plain", "ReST"))
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Title(), 'Plain ReST')
        self.assertEqual(d.Description(), 'Look, Ma, no headers!')
        self.assertEqual(len(d.Subject()), 2)
        self.assertTrue('plain' in d.Subject())
        self.assertTrue('ReST' in d.Subject())

        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.Title(), 'Plain ReST')
        self.assertEqual(d.Description(), 'Look, Ma, no headers!')
        self.assertEqual(len(d.Subject()), 2)
        self.assertTrue('plain' in d.Subject())
        self.assertTrue('ReST' in d.Subject())

    def test_STX_NoHeaders_but_colon(self):
        d = self._makeOne('foo')
        d.editMetadata(title="Plain STX",
                       description="Look, Ma, no headers!",
                       subject=("plain", "STX"))

        d.edit(text_format='structured-text', text=STX_NO_HEADERS_BUT_COLON)
        self.assertEqual(d.EditableBody(), STX_NO_HEADERS_BUT_COLON)

    def test_ReST_NoHeaders_but_colon(self):
        from Products.CMFDefault.Document import REST_AVAILABLE
        if not REST_AVAILABLE:
            return
        d = self._makeOne('foo')
        d.editMetadata(title="Plain ReST",
                       description="Look, Ma, no headers!",
                       subject=("plain", "ST"))

        d.edit(text_format='restructured-text', text=ReST_NO_HEADERS_BUT_COLON)
        self.assertEqual(d.EditableBody(), ReST_NO_HEADERS_BUT_COLON)

    def test_ZMI_edit(self):
        d = self._makeOne('foo')
        d.editMetadata(title="Plain STX",
                       description="Look, Ma, no headers!",
                       subject=("plain", "STX"))

        d.manage_editDocument(text_format='structured-text'
                             , text=STX_NO_HEADERS_BUT_COLON)
        self.assertEqual(d.EditableBody(), STX_NO_HEADERS_BUT_COLON)

    def test_Format_methods(self):
        d = self._makeOne('foo')
        d.setFormat('plain')
        self.assertEqual(d.text_format, 'plain')
        self.assertEqual(d.Format(), 'text/plain')
        d.setFormat(d.Format())
        self.assertEqual(d.text_format, 'plain')

        d.setFormat('structured-text')
        self.assertEqual(d.text_format, 'structured-text')
        self.assertEqual(d.Format(), 'text/plain')
        d.setFormat(d.Format())
        self.assertEqual(d.text_format, 'structured-text')

        from Products.CMFDefault.Document import REST_AVAILABLE
        if REST_AVAILABLE:
            d.setFormat('restructured-text')
            self.assertEqual(d.text_format, 'restructured-text')
            self.assertEqual(d.Format(), 'text/plain')
            d.setFormat(d.Format())
            self.assertEqual(d.text_format, 'restructured-text')

        d.setFormat('html')
        self.assertEqual(d.text_format, 'html')
        self.assertEqual(d.Format(), 'text/html')
        d.setFormat(d.Format())
        self.assertEqual(d.text_format, 'html')

        d.setFormat('foo')
        self.assertEqual(d.text_format, 'structured-text')

    def test_default_format(self):
        d = self._makeOne('foo', text='')

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.text_format, 'structured-text')

    def test_normalize_linebreaks_incoming(self):
        # New feature: Line breaks can be normalized on the way in
        # and on the way out, *if* a specific utility exists.
        # (http://www.zope.org/Collectors/CMF/174)
        d = self._makeOne('foo', text='')
        LF_TEXT = 'This\nis\nsome\ntext'
        CRLF_TEXT = 'This\r\nis\r\nsome\r\ntext'

        # First case, no normalizer installed. Text will stay the same.
        d._edit(LF_TEXT)
        self.assertEqual(d.text, LF_TEXT)

        d._edit(CRLF_TEXT)
        self.assertEqual(d.text, CRLF_TEXT)

        # Now register a normalizer that always replaces CRLF with LF
        # When I pass in CRLFs, the resulting text will only have LFs.
        crlf_to_lf = DummyLinebreakNormalizer('\r\n', '\n')
        sm = getSiteManager()
        sm.registerUtility(crlf_to_lf, ILinebreakNormalizer)

        d._edit(CRLF_TEXT)
        self.assertEqual(d.text, LF_TEXT)

        d._edit(LF_TEXT)
        self.assertEqual(d.text, LF_TEXT)

        # cleanup
        sm.unregisterUtility(crlf_to_lf, ILinebreakNormalizer)

    def test_normalize_linebreaks_outgoing(self):
        # New feature: Line breaks can be normalized on the way in
        # and on the way out, *if* a specific utility exists.
        # (http://www.zope.org/Collectors/CMF/174)
        d = self._makeOne('foo', text='')
        LF_TEXT = 'This\nis\nsome\ntext'
        CRLF_TEXT = 'This\r\nis\r\nsome\r\ntext'
        HEADERS = """Title: \r\nSubject: \r\nPublisher: No publisher\r\nDescription: \r\nContributors: \r\nEffective_date: None\r\nExpiration_date: None\r\nType: Unknown\r\nFormat: text/plain\r\nLanguage: \r\nRights: \r\nSafetyBelt: \r\n\r\n"""

        # First case, no normalizer installed. Text will stay the same.
        d.text = LF_TEXT
        self.assertEqual(d.manage_FTPget(), HEADERS + LF_TEXT)

        d.text = CRLF_TEXT
        self.assertEqual(d.manage_FTPget(), HEADERS + CRLF_TEXT)

        # Now register a normalizer that always replaces CRLF with LF
        # When I pass in CRLFs, the resulting text will only have LFs.
        crlf_to_lf = DummyLinebreakNormalizer('\r\n', '\n')
        sm = getSiteManager()
        sm.registerUtility(crlf_to_lf, ILinebreakNormalizer)

        d.text = (CRLF_TEXT)
        self.assertEqual(d.manage_FTPget(), HEADERS + LF_TEXT)

        d.text = LF_TEXT
        self.assertEqual(d.manage_FTPget(), HEADERS + LF_TEXT)

        # cleanup
        sm.unregisterUtility(crlf_to_lf, ILinebreakNormalizer)


class DocumentFTPGetTests(TransactionalTestBase):

    def setUp(self):
        TransactionalTest.setUp(self)
        self.site = DummySite('site').__of__(self.app)

    def testHTML(self):
        self.REQUEST['BODY'] = BASIC_HTML

        ttool = TypesTool()
        fti = FTIDATA_CMF[0].copy()
        del fti['id']
        ttool._setObject('Document', FTI('Document', **fti))

        ps = self.site._setObject('source_html',
                                  PythonScript('source_html'))
        zpt = self.site._setObject('source_html',
                                   ZopePageTemplate('source_html_template'))
        dir = abspath(dirname(utils.__file__))
        _file = path_join(dir, 'skins', 'zpt_content', 'source_html.py')
        data = open(_file, 'r').read()
        ps.write(data)
        _file = path_join(dir, 'skins', 'zpt_content',
                          'source_html_template.pt')
        data = open(_file, 'r').read()
        zpt.write(data)

        d = self._makeOne('foo')
        d._setPortalTypeName('Document')
        d.PUT(self.REQUEST, self.RESPONSE)

        rnlinesplit = compile(r'\r?\n?')
        simple_lines = rnlinesplit.split(BASIC_HTML)
        get_lines = rnlinesplit.split(d.manage_FTPget())

        # strip off headers
        meta_pattern = compile(r'meta name="([a-z]*)" '
                                 + r'content="([a-z]*)"'
                                 )
        title_pattern = compile(r'<title>(.*)</title>')
        simple_headers = []
        while simple_lines and simple_lines[0] != '<BODY>':
            header = simple_lines[0].strip().lower()
            match = meta_pattern.search(header)
            if match:
                simple_headers.append(match.groups())
            else:
                match = title_pattern.search(header)
                if match:
                    simple_headers.append(('title', match.group(1)))
            simple_lines = simple_lines[1:]

        get_headers = []
        while get_lines and get_lines[0] != '<BODY>':
            header = get_lines[0].strip().lower()
            match = meta_pattern.search(header)
            if match:
                get_headers.append(match.groups())
            else:
                match = title_pattern.search(header)
                if match:
                    get_headers.append(('title', match.group(1)))
            get_lines = get_lines[1:]

        self.assertEqual(get_lines, simple_lines)

        self.assertTrue(get_headers)
        self.assertTrue(simple_headers)
        self.assertTrue(len(get_headers) >= len(simple_headers))

        for header in simple_headers:
            self.assertTrue(header in get_headers)

        body1 = d.EditableBody()
        self.REQUEST['BODY'] = d.manage_FTPget()
        d.PUT(self.REQUEST, self.RESPONSE)
        self.assertEqual(d.EditableBody(), body1)

    def testSTX(self):
        self.REQUEST['BODY'] = SIMPLE_STRUCTUREDTEXT
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        rnlinesplit = compile(r'\r?\n?')

        get_text = d.manage_FTPget()
        simple_lines = rnlinesplit.split(SIMPLE_STRUCTUREDTEXT)
        get_lines = rnlinesplit.split(get_text)

        # strip off headers
        simple_headers = []
        while simple_lines and simple_lines[0]:
            simple_headers.append(simple_lines[0])
            simple_lines = simple_lines[1:]

        get_headers = []
        while get_lines and get_lines[0]:
            get_headers.append(get_lines[0])
            get_lines = get_lines[1:]

        self.assertEqual(get_lines, simple_lines)

        for header in simple_headers:
            self.assertTrue(header in get_headers)


class DocumentPUTTests(TransactionalTestBase):

    def test_PUTBasicHTML(self):
        self.REQUEST['BODY'] = BASIC_HTML
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertTrue(hasattr(d, 'cooked_text'))
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.title, 'Title in tag')
        self.assertEqual(d.text.find('</body>'), -1)
        self.assertEqual(d.Description(), 'Describe me')
        self.assertEqual(len(d.Contributors()), 3)
        self.assertEqual(d.Contributors()[-1],
                         'Benotz, Larry J (larry@benotz.stuff)')
        self.assertEqual(d.cooked_text, '\n  <h1>Not a lot here</h1>\n ')
        self.assertEqual(r.status, 204)

        subj = list(d.Subject())
        self.assertEqual(len(subj), 4)
        subj.sort()
        self.assertEqual(subj, ['content management', 'framework',
                                'unit tests', 'zope'])

    def test_PUTSimpleXHTML(self):
        self.REQUEST['BODY'] = SIMPLE_XHTML
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertTrue(hasattr(d, 'cooked_text'))
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Description(), 'Describe me')
        self.assertEqual(d.cooked_text, '\n  <h1>Not a lot here</h1>\n ')
        self.assertEqual(r.status, 204)

    def test_PutStructuredTextWithHTML(self):
        self.REQUEST['BODY'] = STX_WITH_HTML
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(r.status, 204)

    def test_PutStructuredText(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(r.status, 204)

    def test_PutReStructuredTextWithHTML(self):
        self.REQUEST['BODY'] = ReST_WITH_HTML
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(r.status, 204)

    def test_PutReStructuredText(self):
        self.REQUEST['BODY'] = BASIC_ReST
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(r.status, 204)

    def test_PutHtmlWithDoctype(self):
        html = '%s\n\n  \n   %s' % (DOCTYPE, BASIC_HTML)
        self.REQUEST['BODY'] = html
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Description(), 'Describe me')
        self.assertEqual(r.status, 204)

    def test_PutHtmlWithoutMetadata(self):
        html = HTML_TEMPLATE % {'title': 'Foo', 'body': 'Bar'}
        self.REQUEST['BODY'] = html
        d = self._makeOne('foo')
        r = d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), 'Foo')
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.Description(), '')
        self.assertEqual(d.Subject(), ())
        self.assertEqual(d.Contributors(), ())
        self.assertEqual(d.EffectiveDate(), 'None')
        self.assertEqual(d.ExpirationDate(), 'None')
        self.assertEqual(d.Language(), '')
        self.assertEqual(d.Rights(), '')
        self.assertEqual(r.status, 204)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DocumentTests),
        unittest.makeSuite(DocumentFTPGetTests),
        unittest.makeSuite(DocumentPUTTests),
        ))



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Makers for z3c.recipe.i18n.
"""
import os

def tal_strings_html(path, domain, include_default_domain, exclude_dirs, **kw):
    from zope.app.locales.extract import tal_strings

    return tal_strings(path, domain, include_default_domain,
                       exclude=exclude_dirs, filePattern='*.html')

def tal_strings_xml(path, domain, include_default_domain, exclude_dirs, **kw):
    from zope.app.locales.extract import tal_strings

    return tal_strings(path, domain, include_default_domain,
                       exclude=exclude_dirs, filePattern='*.xml')

def manual_pot(path, domain, **kw):
    catalog = {}
    manual_file = os.path.join(path, 'locales', domain+'-manual.pot')
    if os.path.exists(manual_file):
        manual = file(manual_file, 'r')
        for i, line in enumerate(manual):
            if line.startswith('msgid'):
                catalog[line[5:].strip().strip('"')] = [(manual_file, i+1)]
        manual.close()
    return catalog



##parameters=
##title=Disable Syndication for a resource
##
from Products.CMFCore.utils import getUtilityByInterfaceName

syndtool = getUtilityByInterfaceName('Products.CMFCore.interfaces.ISyndicationTool')
if syndtool.isSyndicationAllowed(context):
    syndtool.disableSyndication(context)
    return context.REQUEST.RESPONSE.redirect(context.absolute_url() +
               '/synPropertiesForm?portal_status_message=Syndication+Disabled')
else:
    return context.REQUEST.RESPONSE.redirect(context.absolute_url() +
            '/synPropertiesForm?portal_status_message=Syndication+Not+Allowed')



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""CMFDefault upgrade steps.

$Id: __init__.py 110420 2010-04-01 17:12:56Z tseaver $
"""

def mark_as_finished(tool):
    """empty upgrade step handler
    """



##parameters=change='', change_and_view=''
##
from Products.CMFDefault.utils import decode
from Products.CMFDefault.utils import Message as _

form = context.REQUEST.form
if change and \
        context.image_edit_control(**form) and \
        context.setRedirect(context, 'object/edit'):
    return
elif change_and_view and \
        context.image_edit_control(**form) and \
        context.setRedirect(context, 'object/view'):
    return


options = {}

options['title'] = context.Title()
options['description'] = context.Description()
options['format'] = context.Format()

buttons = []
target = context.getActionInfo('object/edit')['url']
buttons.append( {'name': 'change', 'value': _(u'Change')} )
buttons.append( {'name': 'change_and_view', 'value': _(u'Change and View')} )
options['form'] = { 'action': target,
                    'listButtonInfos': tuple(buttons) }

return context.image_edit_template(**decode(options, script))



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Upgrade tests.
"""

import unittest
from Testing import ZopeTestCase

import transaction
from os.path import abspath
from os.path import dirname
from os.path import join as path_join

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.User import UnrestrictedUser
from Products.CMFCore.tests.base.testcase import WarningInterceptor
from Products.CMFDefault.testing import FunctionalLayer
from Products.GenericSetup.context import TarballImportContext
from zope.site.hooks import setSite

here = abspath(dirname(__file__))


class FunctionalUpgradeTestCase(ZopeTestCase.FunctionalTestCase,
                                WarningInterceptor):

    layer = FunctionalLayer
    _setup_fixture = 0

    def afterSetUp(self):
        zexp_path = path_join(here, '%s.zexp' % self._SITE_ID)
        self._trap_warning_output()
        self.app._importObjectFromFile(zexp_path, verify=0)
        self._free_warning_output()
        transaction.commit()

    def beforeTearDown(self):
        self.app._delObject(self._SITE_ID)
        transaction.commit()

    def testUpgradeAllProposed(self):
        request = self.app.REQUEST
        oldsite = getattr(self.app, self._SITE_ID)
        stool = oldsite.portal_setup
        profile_id = 'Products.CMFDefault:default'
        upgrades = []
        for upgrade_info in stool.listUpgrades(profile_id):
            if isinstance(upgrade_info, list):
                for info in upgrade_info:
                    if info['proposed']:
                        upgrades.append(info['id'])
                continue
            if upgrade_info['proposed']:
                upgrades.append(upgrade_info['id'])

        request.form['profile_id'] = profile_id
        request.form['upgrades'] = upgrades
        stool.manage_doUpgrades(request)

        self.assertEqual(stool.getLastVersionForProfile(profile_id),
                         ('2', '3'))

        newSecurityManager(None, UnrestrictedUser('god', '', ['Manager'], ''))
        setSite(self.app.site)
        expected_export = self.app.site.portal_setup.runAllExportSteps()
        setSite(oldsite)
        upgraded_export = stool.runAllExportSteps()

        expected = TarballImportContext(stool, expected_export['tarball'])
        upgraded = TarballImportContext(stool, upgraded_export['tarball'])
        diff = stool.compareConfigurations(upgraded, expected)
        self.assertEqual(diff, '', diff)


class UpgradeFrom20Tests(FunctionalUpgradeTestCase):

    _SITE_ID = 'cmf20Site'


class UpgradeFrom21Tests(FunctionalUpgradeTestCase):

    _SITE_ID = 'cmf21Site'


class UpgradeFrom22Tests(FunctionalUpgradeTestCase):

    _SITE_ID = 'cmf22Site'


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(UpgradeFrom20Tests),
        unittest.makeSuite(UpgradeFrom21Tests),
        unittest.makeSuite(UpgradeFrom22Tests),
        ))



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Formlib schema vocabulary base classes.

$Id: vocabulary.py 110420 2010-04-01 17:12:56Z tseaver $
"""

from zope.interface import implements
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary


class SimpleVocabulary(SimpleVocabulary):

    def fromTitleItems(cls, items, *interfaces):
        """Construct a vocabulary from a list of (token, value, title) tuples.
        """
        terms = [ cls.createTerm(value, token, title)
                  for (token, value, title) in items ]
        return cls(terms, *interfaces)

    fromTitleItems = classmethod(fromTitleItems)


class StaticVocabulary(object):

    """Vocabulary factory for static items.
    """

    implements(IVocabularyFactory)

    def __init__(self, items):
        self._vocabulary = SimpleVocabulary.fromTitleItems(items)

    def __call__(self, context):
        return self._vocabulary



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
"""

import unittest

from Acquisition import Implicit
from zope.component.testing import PlacelessSetup
from zope.globalrequest import clearRequest
from zope.globalrequest import setRequest

from .test_ursa import DummyActionsTool
from .test_ursa import DummyPropertiesTool
from .test_ursa import DummyResponse
from .test_ursa import DummyURLTool
from Products.CMFCore.interfaces import IActionsTool
from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.interfaces import ITypesTool


class AbsolutIconsTests(unittest.TestCase, PlacelessSetup):

    def setUp(self):
        PlacelessSetup.setUp(self)

    def tearDown(self):
        clearRequest()
        PlacelessSetup.tearDown(self)

    def _getTargetClass(self):
        from Products.CMFDefault.browser.skins.icons import View
        return View

    def _makeOne(self, site=None):
        if site is None:
            site = self._makeSite()
        request = DummyRequest()
        setRequest(request)
        return self._getTargetClass()(site, request)

    def _makeSite(self, types=None, actions=None):
        from zope.component import getSiteManager
        from Products.CMFCore.interfaces import IPropertiesTool
        site = DummyContext()
        tool = site.portal_properties = DummyPropertiesTool()
        sm = getSiteManager()
        sm.registerUtility(tool, IPropertiesTool)
        if types is not None:
            site.portal_url = DummyURLTool(site)
            sm.registerUtility(DummyTypesTool(types), ITypesTool)
            sm.registerUtility(DummyMembershipTool(), IMembershipTool)
        if actions is not None:
            sm.registerUtility(DummyActionsTool(actions), IActionsTool)
        site.absolute_url = lambda: 'http://example.com'
        return site

    def test_show_icons_not_set(self):
        #Show action icons not set
        view = self._makeOne()
        self.failIf(view._show_icons)

    def test_show_icons_enabled(self):
        #Show actions set to True
        site = self._makeSite()
        site.portal_properties.enable_actionicons = True
        view = self._makeOne(site)
        self.failUnless(view._show_icons)

    def test_show_icons_disabled(self):
        #Show action icons set to False
        site = self._makeSite()
        site.portal_properties.enable_actionicons = False
        view = self._makeOne(site)
        self.failIf(view._show_icons)

    def test_type_icons_with_action_icons_disabled(self):
        #Type actions should always be visible
        types = [DummyType("Document"), DummyType("Image")]
        site = self._makeSite(types=types)
        view = self._makeOne(site)
        self.failIf(view.show_icons)

        css = view.types()
        self.assertEqual(css,""".Document {background: url(http://example.com/Document.png) no-repeat 0.1em}

.Image {background: url(http://example.com/Image.png) no-repeat 0.1em}""")

    def test_type_icons_with_action_icons_enabled(self):
        #Type actions should always be visible"""
        types = [DummyType("Document"), DummyType("Image")]
        site = self._makeSite(types=types)
        site.portal_properties.enable_actionicons = True
        view = self._makeOne(site)
        self.failUnless(view.show_icons)

        css = view.types()
        self.assertEqual(css,""".Document {background: url(http://example.com/Document.png) no-repeat 0.1em}

.Image {background: url(http://example.com/Image.png) no-repeat 0.1em}""")

    def test_action_icons_with_action_icons_disabled(self):
        #Action icons disabled. Image less styles should be returned.
        site = self._makeSite(actions=ACTIONS)
        view = self._makeOne(site)
        self.failIf(view.show_icons)

        css = view.actions()
        self.assertEqual(css, """/* user actions */

.Login {/* Login.png */}

.Logout {/* Logout.png */}

/* object actions */

.Edit {/* Edit.png */}

/* folder actions */

.folderContents {/* folderContents.png */}

/* workflow actions */

.Publish {/* Publish.png */}

/* global actions */

.Undo {/* Undo.png */}""")

    def test_action_icons_with_action_icons_enabled(self):
        #Action icons enabled. Styles with images should be returned.
        site = self._makeSite(actions=ACTIONS)
        site.portal_properties.enable_actionicons = True
        view = self._makeOne(site)
        self.failUnless(view.show_icons)

        css = view.actions()
        self.assertEqual(css, """/* user actions */

.Login {background: url(Login.png) no-repeat 0.1em}

.Logout {background: url(Logout.png) no-repeat 0.1em}

/* object actions */

.Edit {background: url(Edit.png) no-repeat 0.1em}

/* folder actions */

.folderContents {background: url(folderContents.png) no-repeat 0.1em}

/* workflow actions */

.Publish {background: url(Publish.png) no-repeat 0.1em}

/* global actions */

.Undo {background: url(Undo.png) no-repeat 0.1em}""")


class DummyContext(Implicit):

    pass


class DummyRequest:

    def __init__(self):
        self.RESPONSE = DummyResponse()

    def get(self, key, default=None):
        return {}


class DummyType:

    def __init__(self, id):
        from Products.CMFCore.Expression import Expression
        self.id = id
        self.icon_expr_object = Expression('string:${portal_url}/%s.png' % id)

    def getIconExprObject(self):
        return getattr(self, 'icon_expr_object', None)


class DummyTypesTool:

    def __init__(self, types=None):
        if types is None:
            self.typeInfos = []
        else:
            self.typeInfos = types[:]

    def listTypeInfo(self):
        return self.typeInfos


def DummyAction(name):

    return {'id':name, 'icon':'%s.png' % name}


ACTIONS = {'global': [DummyAction('Undo')],
           'user': [DummyAction('Login'), DummyAction('Logout')],
           'object': [DummyAction('Edit')],
           'folder': [DummyAction('folderContents')],
           'workflow': [DummyAction('Publish')],
          }


class DummyMembershipTool:

    def isAnonymousUser(self):
        return True



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFDefault product exceptions.

$Id: exceptions.py 110420 2010-04-01 17:12:56Z tseaver $
"""

from zope.schema import ValidationError
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('cmf_default')

from AccessControl import ModuleSecurityInfo
security = ModuleSecurityInfo('Products.CMFDefault.exceptions')

security.declarePublic('AccessControl_Unauthorized')
from Products.CMFCore.exceptions import AccessControl_Unauthorized

security.declarePublic('BadRequest')
from Products.CMFCore.exceptions import BadRequest

security.declarePublic('CopyError')
from Products.CMFCore.exceptions import CopyError

security.declarePublic('ResourceLockedError')
from Products.CMFCore.exceptions import ResourceLockedError

security.declarePublic('WorkflowException')
from Products.CMFCore.WorkflowCore import WorkflowException

security.declarePublic('zExceptions_Unauthorized')
from Products.CMFCore.exceptions import zExceptions_Unauthorized


security.declarePublic('EditingConflict')
class EditingConflict(Exception):
    """ Editing conflict error.
    """


security.declarePublic('DiscussionNotAllowed')
class DiscussionNotAllowed(Exception):
    """ Discussion not allowed error.
    """


security.declarePublic('IllegalHTML')
class IllegalHTML(ValueError):
    """ Illegal HTML error.
    """


security.declarePublic('MetadataError')
class MetadataError(Exception):
    """ Metadata error.
    """

class EmailAddressInvalid(ValidationError):
    __doc__ = _(u'Invalid email address.')



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Search Form Schema.
"""

from datetime import date
from datetime import timedelta

from DateTime import DateTime
from zope.component import getUtility
from zope.interface import directlyProvides
from zope.interface import Interface
from zope.schema import ASCIILine
from zope.schema import Choice
from zope.schema import List
from zope.schema import TextLine
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

from Products.CMFCore.interfaces import ICatalogTool
from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.interfaces import ITypesTool
from Products.CMFDefault.utils import Message as _

def status_vocab(context):
    """Provides a list of workflow states"""
    ctool = getUtility(ICatalogTool)
    values = [SimpleTerm(None, None, _(u"-- any --"))]
    values += [SimpleTerm(v, v, v)
               for v in ctool.uniqueValuesFor('review_state')]
    return SimpleVocabulary(values)
directlyProvides(status_vocab, IContextSourceBinder)

def subject_vocab(context):
    """Provides a list of subject keywords"""
    ctool = getUtility(ICatalogTool)
    values = [SimpleTerm(None, None, _(u"-- any --"))]
    values += [SimpleTerm(v, v, v)
               for v in ctool.uniqueValuesFor('Subject')]
    return SimpleVocabulary(values)
directlyProvides(subject_vocab, IContextSourceBinder)

def date_vocab(context):
    """Provides a list of dates for searching with"""
    mtool = getUtility(IMembershipTool)
    dates = [SimpleTerm(date(1970, 1, 1), date(1970, 1, 1), _(u'Ever'))]
    if not mtool.isAnonymousUser():
        member = mtool.getAuthenticatedMember()
        login_time = member.getProperty('last_login_time')
        if not hasattr(login_time, 'parts'):
            login_time = DateTime(login_time)
        login = date(*login_time.parts()[:3])
        dates.append(SimpleTerm(
            login, login, _(u'Last login'))
                     )

    today = date.today()
    dates.append(SimpleTerm(today - timedelta(days=1),
                            today - timedelta(days=1),
                            _(u'Yesterday')
                            )
                 )
    dates.append(SimpleTerm(today - timedelta(days=7),
                            today - timedelta(days=7),
                            _(u'Last week')
                            )
                 )
    dates.append(SimpleTerm(today - timedelta(days=31),
                            today - timedelta(days=31),
                            _(u'Last month')
                            )
                 )
    return SimpleVocabulary(dates)
directlyProvides(date_vocab, IContextSourceBinder)

def type_vocab(context):
    """Provides a list of portal types"""
    ttool =  getUtility(ITypesTool)
    types = ttool.listTypeInfo()
    terms = [SimpleTerm(None, None, _(u"-- any --"))]
    terms += [SimpleTerm(t.getId(), t.getId(), t.Title()) for t in types]
    return SimpleVocabulary(terms)
directlyProvides(type_vocab, IContextSourceBinder)


class ISearchSchema(Interface):

    review_state = List(
                        title=_(u"Review Status"),
                        description=_(
                            u"As a reviewer, you may search for items based on"
                            u" their review state. If you wish to constrain"
                            u" results to items in certain states, select them"
                            u" from this list."),
                        value_type=Choice(
                            source=status_vocab
                            ),
                        required=False,
                        )

    SearchableText = TextLine(
                    title=_(u"Full Text"),
                    description=_(
                        u"For a simple text search, enter your search term"
                        u" here. Multiple words may be found by combining them"
                        u" with AND and OR. This will find text in items'"
                        u" contents, title and description."),
                    required=False
                    )

    Title = TextLine(
                title=_(u"Title"),
                required=False
                )

    Subject = List(
                title=_(u"Subject"),
                description=_(u""),
                value_type=Choice(
                    source=subject_vocab
                    ),
                required=False
                )

    Description = TextLine(
                    title=_(u"Description"),
                    description=_(
                        u"You may also search the items' descriptions and"
                        u" titles specifically. Multiple words may be found by"
                        u" combining them with AND and OR."),
                    required=False
                    )

    created = Choice(
            title=_(u"Find new items since..."),
            description=(_(
                u"You may find only recent items by selecting a time-frame."
                        )),
            source=date_vocab,
            default=date.today()
            )

    portal_type = List(
                    title=_(u"Item type"),
                    description=_(
                        u"You may limit your results to particular kinds of"
                        u" items by selecting them above. To find all kinds of"
                        u" items, do not select anything."),
                    value_type=Choice(
                        source=type_vocab,
                        ),
                    required=False,
                    )

    listCreators = ASCIILine(
                title=_(u"Creator"),
                description=_(
                    u"To find items by a particular user only, enter their"
                    u" username above. Note that you must enter their username"
                    u" exactly."),
                required=False
                )



##parameters=tree_root
##title=Standard Tree
##
from ZTUtils import SimpleTreeMaker
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.permissions import ManagePortal
from Products.CMFDefault.utils import decode

mtool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IMembershipTool')

tm = SimpleTreeMaker('tb_tree')
def getKids(object):
    return object.talkback.getReplies()
tm.setChildAccess(function=getKids)

tree, rows = tm.cookieTree(tree_root)
rows.pop(0)

is_delete_allowed = mtool.checkPermission(ManagePortal, context)

items = []
for row in rows:
    branch = row.branch()
    item = row.object
    item_url = item.absolute_url()
    items.append({'tree_colspan': tree.height - row.depth,
                  'tree_icon': branch and branch['img'] or '',
                  'tree_id': row.id,
                  'tree_indent': row.depth - 1,
                  'tree_url': branch and branch['link'] or '',
                  'creators': item.listCreators(),
                  'date': item.CreationDate(),
                  'delete_url': is_delete_allowed and \
                                ('%s/discitem_delete' % item_url) or '',
                  'icon': item.getIconURL(),
                  'title': item.Title(),
                  'url': item_url})

return decode(tuple(items), context)



## Script (Python) "permalink"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Returns an object by unique id
##
from Products.CMFCore.utils import getUtilityByInterfaceName

subpath = traverse_subpath[0]
uid_handler=getUtilityByInterfaceName('Products.CMFUid.interfaces.IUniqueIdHandler')

# appending 'isAvailable' instead of a unique id returns if
# the site permalink feature is available.
if str(subpath).strip() == 'isAvailable':
    # no permalink feature without an uid handler tool being installed
    if uid_handler is None:
        return '0'
    ptool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IPropertiesTool')
    isAvailable = getattr(ptool, 'enable_permalink', 0)
    return str(int(isAvailable))

obj = uid_handler.getObject(subpath)
# workarround for an acquisition wrapping problem.
obj = context.restrictedTraverse(obj.getPhysicalPath())

ti = obj.getTypeInfo()
method_id = ti and ti.queryMethodID('view', context=obj)
if method_id:
    return getattr(obj, method_id)()
return obj()



##############################################################################
#
# Copyright (c) 2010 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Search form tests"""

import unittest

from zope.component import getSiteManager
from zope.testing.cleanup import cleanUp

from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyTool
from Products.CMFDefault.browser.test_utils import DummyRequest


class SearchFormTests(unittest.TestCase):

    def setUp(self):
        self.site = DummySite('site')
        getSiteManager().registerUtility(DummyTool(), IMembershipTool)

    def tearDown(self):
        cleanUp()

    def _getTargetClass(self):
        from Products.CMFDefault.browser.search.search import Search
        return Search(self.site, DummyRequest())

    def test_is_anonymous(self):
        view = self._getTargetClass()
        self.assertTrue(view.is_anonymous)
        self.assertEqual(view.search_fields.get('review_state'), None)

    def test_is_not_anonymous(self):
        view = self._getTargetClass()
        getSiteManager().getUtility(IMembershipTool).anon = 0
        self.assertFalse(view.is_anonymous)
        self.assertNotEqual(view.search_fields.get('review_state'), None)

    def test_strip_unused_paramaters(self):
        view = self._getTargetClass()
        data = {'portal_type': ['Document'], 'review_state':u"None",
                'Subject':u"None"}
        view.handle_search('search', data)
        self.assertEqual(view._query, {'portal_type':['Document']})

    def test_add_search_vars_to_hidden(self):
        view = self._getTargetClass()
        self.assertFalse(hasattr(view, '_query'))
        data = {'portal_type': ['Document']}
        view.handle_search('search', data)
        self.assertEqual(view._getHiddenVars(), data)

    def test_search_returns_results(self):
        view = self._getTargetClass()
        self.assertNotEqual(view.template, view.results)
        view.handle_search('search', {})
        self.assertEqual(view.template.filename, view.results.filename)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SearchFormTests))
    return suite



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Allow the "view" of a folder to be skinned by type.

$Id: SkinnedFolder.py 110420 2010-04-01 17:12:56Z tseaver $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from zope.component.factory import Factory
from zope.interface import implements

from Products.CMFCore.CMFCatalogAware import CatalogAware
from Products.CMFCore.CMFCatalogAware import WorkflowAware
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.permissions import ModifyPortalContent
from Products.CMFDefault.permissions import View


class SkinnedFolder(CatalogAware, WorkflowAware, PortalFolder):

    """ Skinned Folder class. 
    """

    implements(IContentish)

    security = ClassSecurityInfo()

    manage_options = PortalFolder.manage_options + WorkflowAware.manage_options

    # XXX: maybe we should subclass from DefaultDublinCoreImpl or refactor it

    security.declarePrivate('notifyModified')
    def notifyModified(self):
        """ Take appropriate action after the resource has been modified.

        Update creators.
        """
        self.addCreator()

    security.declareProtected(ModifyPortalContent, 'addCreator')
    addCreator = DefaultDublinCoreImpl.addCreator.im_func

    security.declareProtected(View, 'listCreators')
    listCreators = DefaultDublinCoreImpl.listCreators.im_func

    security.declareProtected(View, 'Creator')
    Creator = DefaultDublinCoreImpl.Creator.im_func

    #
    #   'IContentish' interface method
    #
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """
        SeachableText is used for full text seraches of a portal.  It
        should return a concatenation of all useful text.
        """
        return "%s %s" % (self.title, self.description)

InitializeClass(SkinnedFolder)

SkinnedFolderFactory = Factory(SkinnedFolder)

def addSkinnedFolder( self, id, title='', description='', REQUEST=None ):
    """
    """
    sf = SkinnedFolder( id, title )
    sf.description = description
    self._setObject(id, sf, suppress_events=True)
    sf = self._getOb( id )
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( sf.absolute_url() + '/manage_main' )



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFDefault portal_registration tool.
"""

from AccessControl.requestmethod import postonly
from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import aq_base
from Acquisition import aq_chain
from App.class_init import InitializeClass
from Products.MailHost.interfaces import IMailHost
from zope.component import getUtility
from zope.schema import ValidationError
from ZPublisher.BaseRequest import RequestContainer

from Products.CMFCore.interfaces import IMembershipTool
from Products.CMFCore.RegistrationTool import RegistrationTool as BaseTool
from Products.CMFCore.utils import _checkPermission
from Products.CMFDefault.permissions import ManagePortal
from Products.CMFDefault.utils import checkEmailAddress
from Products.CMFDefault.utils import Message as _


class RegistrationTool(BaseTool):

    """ Manage through-the-web signup policies.
    """

    meta_type = 'Default Registration Tool'

    security = ClassSecurityInfo()

    def _getValidEmailAddress(self, member):
        email = member.getProperty('email')

        # assert that we can actually get an email address, otherwise
        # the template will be made with a blank To:, this is bad
        if email is None:
            msg = _(u'No email address is registered for member: '
                    u'${member_id}', mapping={'member_id': member.getId()})
            raise ValueError(msg)

        checkEmailAddress(email)
        return email

    #
    #   'portal_registration' interface
    #
    security.declarePublic( 'testPasswordValidity' )
    def testPasswordValidity(self, password, confirm=None):

        """ Verify that the password satisfies the portal's requirements.

        o If the password is valid, return None.
        o If not, return a string explaining why.
        """
        if not password:
            return _(u'You must enter a password.')

        if len(password) < 5 and not _checkPermission(ManagePortal, self):
            return _(u'Your password must contain at least 5 characters.')

        if confirm is not None and confirm != password:
            return _(u'Your password and confirmation did not match. '
                     u'Please try again.')

        return None

    security.declarePublic( 'testPropertiesValidity' )
    def testPropertiesValidity(self, props, member=None):

        """ Verify that the properties supplied satisfy portal's requirements.

        o If the properties are valid, return None.
        o If not, return a string explaining why.
        """
        if member is None: # New member.

            username = props.get('username', '')
            if not username:
                return _(u'You must enter a valid name.')

            if not self.isMemberIdAllowed(username):
                return _(u'The login name you selected is already in use or '
                         u'is not valid. Please choose another.')

            email = props.get('email')
            if email is None:
                return _(u'You must enter an email address.')

            try:
                checkEmailAddress(email)
            except ValidationError:
                return _(u'You must enter a valid email address.')

        else: # Existing member.
            email = props.get('email')

            if email is not None:
                try:
                    checkEmailAddress(email)
                except ValidationError:
                    return _(u'You must enter a valid email address.')

            # Not allowed to clear an existing non-empty email.
            existing = member.getProperty('email')

            if existing and email == '':
                return _(u'You must enter a valid email address.')

        return None

    security.declarePublic('mailPassword')
    def mailPassword(self, forgotten_userid, REQUEST):
        """ Email a forgotten password to a member.

        o Raise an exception if user ID is not found.
        """
        mtool = getUtility(IMembershipTool)
        member = mtool.getMemberById(forgotten_userid)

        if member is None:
            raise ValueError(_(u'The username you entered could not be '
                               u'found.'))

        email = self._getValidEmailAddress(member)

        # Rather than have the template try to use the mailhost, we will
        # render the message ourselves and send it from here (where we
        # don't need to worry about 'UseMailHost' permissions).
        if getattr(self, 'REQUEST', None) is None:
            context = RequestContainer(REQUEST=REQUEST)
            for item in reversed(aq_chain(self)):
                context = aq_base(item).__of__(context)
        else:
            context = self
        method = context.password_email
        kw = {'member': member, 'password': member.getPassword()}

        if getattr(aq_base(method), 'isDocTemp', 0):
            mail_text = method(self, REQUEST, **kw)
        else:
            mail_text = method(**kw)

        host = getUtility(IMailHost)
        host.send(mail_text)

        return context.mail_password_response(self, REQUEST)

    security.declarePublic('registeredNotify')
    def registeredNotify(self, new_member_id, password=None, REQUEST=None):
        """ Handle mailing the registration / welcome message.
        """
        if REQUEST is None:
            raise ValueError(u"'REQUEST' argument is missing.")

        mtool = getUtility(IMembershipTool)
        member = mtool.getMemberById(new_member_id)

        if member is None:
            raise ValueError(_(u'The username you entered could not be '
                               u'found.'))

        if password is None:
            password = member.getPassword()

        email = self._getValidEmailAddress(member)

        # Rather than have the template try to use the mailhost, we will
        # render the message ourselves and send it from here (where we
        # don't need to worry about 'UseMailHost' permissions).
        if getattr(self, 'REQUEST', None) is None:
            context = RequestContainer(REQUEST=REQUEST)
            for item in reversed(aq_chain(self)):
                context = aq_base(item).__of__(context)
        else:
            context = self
        method = context.registered_email
        kw = {'member': member, 'password': password, 'email': email}

        if getattr(aq_base(method), 'isDocTemp', 0):
            mail_text = method(self, REQUEST, **kw)
        else:
            mail_text = method(**kw)

        host = getUtility(IMailHost)
        host.send(mail_text)

    security.declareProtected(ManagePortal, 'editMember')
    @postonly
    def editMember(self, member_id, properties=None, password=None,
                   roles=None, domains=None, REQUEST=None):
        """ Edit a user's properties and security settings

        o Checks should be done before this method is called using
          testPropertiesValidity and testPasswordValidity
        """
        mtool = getUtility(IMembershipTool)
        member = mtool.getMemberById(member_id)
        member.setMemberProperties(properties)
        member.setSecurityProfile(password, roles, domains)

        return member

InitializeClass(RegistrationTool)



##parameters=b_start=0, member_id='', member_email='', password='', confirm='', send_password='', add='', cancel=''
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.permissions import ManageUsers
from Products.CMFDefault.utils import decode
from Products.CMFDefault.utils import Message as _

atool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IActionsTool')
mtool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IMembershipTool')
ptool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IPropertiesTool')
utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')
portal_url = utool()
validate_email = ptool.getProperty('validate_email')
is_anon = mtool.isAnonymousUser()
is_newmember = False
is_usermanager = mtool.checkPermission(ManageUsers, mtool)


form = context.REQUEST.form
if add and \
        context.validatePassword(**form) and \
        context.members_add_control(**form) and \
        context.setRedirect(atool, ('global/members_register', 'user/login'),
                            b_start=b_start):
    return
elif cancel and \
        context.setRedirect(atool, 'global/manage_members', b_start=b_start):
    return


options = {}

if context.REQUEST.get('is_newmember', False) == True:
    is_anon = False
    is_newmember = True

options['title'] = is_usermanager and _(u'Register a New Member') \
                                  or _(u'Become a Member')
options['member_id'] = member_id
options['member_email'] = member_email
options['password'] = is_newmember and context.REQUEST.get('password', '') or ''
options['send_password'] = send_password
options['portal_url'] = portal_url
options['isAnon'] = is_anon
options['isAnonOrUserManager'] = is_anon or is_usermanager
options['isNewMember'] = is_newmember
options['isOrdinaryMember'] = not (is_anon or is_newmember or is_usermanager)
options['validate_email'] = validate_email

buttons = []
if is_newmember:
    target = atool.getActionInfo('user/logged_in')['url']
    buttons.append( {'name': 'login', 'value': _(u'Log in')} )
else:
    target = atool.getActionInfo(('global/members_register',
                                  'user/join'))['url']
    buttons.append( {'name': 'add', 'value': _(u'Register')} )
    buttons.append( {'name': 'cancel', 'value': _(u'Cancel')} )
options['form'] = { 'action': target,
                    'listButtonInfos': tuple(buttons) }

return context.join_template(**decode(options, script))



##parameters=change_type
##title=Set local roles
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import Message as _

mtool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IMembershipTool')

if change_type == 'add':
    mtool.setLocalRoles(obj=context,
                        member_ids=context.REQUEST.get('member_ids', ()),
                        member_role=context.REQUEST.get('member_role', ''),
                        REQUEST=context.REQUEST)
else:
    mtool.deleteLocalRoles(obj=context,
                           member_ids=context.REQUEST.get('member_ids', ()),
                           REQUEST=context.REQUEST)

context.setStatus(True, _(u'Local Roles changed.'))
context.setRedirect(context, 'object/localroles')



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Dublin Core support for content types.
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from Acquisition import aq_base
from App.class_init import InitializeClass
from App.special_dtml import DTMLFile
from DateTime.DateTime import DateTime
from OFS.PropertyManager import PropertyManager
from zope.component import queryUtility
from zope.interface import implements

from Products.CMFCore.interfaces import ICatalogableDublinCore
from Products.CMFCore.interfaces import IDublinCore
from Products.CMFCore.interfaces import IMetadataTool
from Products.CMFCore.interfaces import IMutableDublinCore
from Products.CMFDefault.permissions import ModifyPortalContent
from Products.CMFDefault.permissions import View
from Products.CMFDefault.utils import _dtmldir
from Products.CMFDefault.utils import semi_split
from Products.CMFDefault.utils import tuplize

_marker=[]

# For http://www.zope.org/Collectors/CMF/325
# We only really need this once, at startup.
_zone = DateTime().timezone()


class DefaultDublinCoreImpl( PropertyManager ):

    """ Mix-in class which provides Dublin Core methods.
    """

    implements(IDublinCore, ICatalogableDublinCore, IMutableDublinCore)

    security = ClassSecurityInfo()

    def __init__( self
                , title=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format='text/html'
                , language=''
                , rights=''
                ):
        now = DateTime()
        self.creation_date = now
        self.modification_date = now
        self.creators = ()
        self._editMetadata( title
                          , subject
                          , description
                          , contributors
                          , effective_date
                          , expiration_date
                          , format
                          , language
                          , rights
                          )

    #
    #  Set-modification-date-related methods.
    #  In DefaultDublinCoreImpl for lack of a better place.
    #

    # Class variable default for an upgrade.
    modification_date = None

    security.declarePrivate('notifyModified')
    def notifyModified(self):
        """ Take appropriate action after the resource has been modified.

        Update creators and modification_date.
        """
        self.addCreator()
        self.setModificationDate()

    security.declareProtected(ModifyPortalContent, 'addCreator')
    def addCreator(self, creator=None):
        """ Add creator to Dublin Core creators.
        """
        if creator is None:
            user = getSecurityManager().getUser()
            creator = user and user.getId()

        # call self.listCreators() to make sure self.creators exists
        if creator and not creator in self.listCreators():
            self.creators = self.creators + (creator, )

    security.declareProtected(ModifyPortalContent, 'setModificationDate')
    def setModificationDate(self, modification_date=None):
        """ Set the date when the resource was last modified.

        When called without an argument, sets the date to now.
        """
        if modification_date is None:
            self.modification_date = DateTime()
        else:
            self.modification_date = self._datify(modification_date)

    #
    #  DublinCore interface query methods
    #
    security.declareProtected(View, 'Title')
    def Title( self ):
        """ Dublin Core Title element - resource name.
        """
        return self.title

    security.declareProtected(View, 'listCreators')
    def listCreators(self):
        """ List Dublin Core Creator elements - resource authors.
        """
        if not hasattr(aq_base(self), 'creators'):
            # for content created with CMF versions before 1.5
            owner_tuple = self.getOwnerTuple()
            if owner_tuple:
                self.creators = (owner_tuple[1],)
            else:
                self.creators = ()
        return self.creators

    security.declareProtected(View, 'Creator')
    def Creator(self):
        """ Dublin Core Creator element - resource author.
        """
        creators = self.listCreators()
        return creators and creators[0] or ''

    security.declareProtected(View, 'Subject')
    def Subject( self ):
        """ Dublin Core Subject element - resource keywords.
        """
        return getattr( self, 'subject', () ) # compensate for *old* content

    security.declareProtected(View, 'Description')
    def Description( self ):
        """ Dublin Core Description element - resource summary.
        """
        return self.description

    security.declareProtected(View, 'Publisher')
    def Publisher(self):
        """ Dublin Core Publisher element - resource publisher.
        """
        tool = queryUtility(IMetadataTool)
        if tool is not None:
            return tool.getPublisher()

        return 'No publisher'

    security.declareProtected(View, 'listContributors')
    def listContributors(self):
        """ Dublin Core Contributor elements - resource collaborators.
        """
        return self.contributors

    security.declareProtected(View, 'Contributors')
    def Contributors(self):
        """ Deprecated alias of listContributors.
        """
        return self.listContributors()

    security.declareProtected(View, 'Date')
    def Date( self, zone=None ):
        """ Dublin Core Date element - default date.
        """
        if zone is None:
            zone = _zone
        # Return effective_date if set, modification date otherwise
        date = getattr(self, 'effective_date', None )
        if date is None:
            date = self.modified()
        return date.toZone(zone).ISO()

    security.declareProtected(View, 'CreationDate')
    def CreationDate( self, zone=None ):
        """ Dublin Core Date element - date resource created.
        """
        if zone is None:
            zone = _zone
        # return unknown if never set properly
        if self.creation_date:
            return self.creation_date.toZone(zone).ISO()
        else:
            return 'Unknown'

    security.declareProtected(View, 'EffectiveDate')
    def EffectiveDate( self, zone=None ):
        """ Dublin Core Date element - date resource becomes effective.
        """
        if zone is None:
            zone = _zone
        ed = getattr( self, 'effective_date', None )
        return ed and ed.toZone(zone).ISO() or 'None'

    security.declareProtected(View, 'ExpirationDate')
    def ExpirationDate( self, zone=None ):
        """ Dublin Core Date element - date resource expires.
        """
        if zone is None:
            zone = _zone
        ed = getattr( self, 'expiration_date', None )
        return ed and ed.toZone(zone).ISO() or 'None'

    security.declareProtected(View, 'ModificationDate')
    def ModificationDate( self, zone=None ):
        """ Dublin Core Date element - date resource last modified.
        """
        if zone is None:
            zone = _zone
        return self.modified().toZone(zone).ISO()

    security.declareProtected(View, 'Type')
    def Type( self ):
        """ Dublin Core Type element - resource type.
        """
        ti = self.getTypeInfo()
        return ti is not None and ti.Title() or 'Unknown'

    security.declareProtected(View, 'Format')
    def Format( self ):
        """ Dublin Core Format element - resource format.
        """
        return self.format

    security.declareProtected(View, 'Identifier')
    def Identifier( self ):
        """ Dublin Core Identifier element - resource ID.
        """
        # XXX: fixme using 'portal_metadata' (we need to prepend the
        #      right prefix to self.getPhysicalPath().
        return self.absolute_url()

    security.declareProtected(View, 'Language')
    def Language( self ):
        """ Dublin Core Language element - resource language.
        """
        return self.language

    security.declareProtected(View, 'Rights')
    def Rights( self ):
        """ Dublin Core Rights element - resource copyright.
        """
        return self.rights

    #
    #  DublinCore utility methods
    #
    def content_type( self ):
        """ WebDAV needs this to do the Right Thing (TM).
        """
        return self.Format()

    __FLOOR_DATE = DateTime( 1970, 0 ) # always effective

    security.declareProtected(View, 'isEffective')
    def isEffective( self, date ):
        """ Is the date within the resource's effective range?
        """
        pastEffective = ( self.effective_date is None
                       or self.effective_date <= date )
        beforeExpiration = ( self.expiration_date is None
                          or self.expiration_date >= date )
        return pastEffective and beforeExpiration

    #
    #  CatalogableDublinCore methods
    #
    security.declareProtected(View, 'created')
    def created( self ):
        """ Dublin Core Date element - date resource created.
        """
        # allow for non-existent creation_date, existed always
        date = getattr( self, 'creation_date', None )
        return date is None and self.__FLOOR_DATE or date

    security.declareProtected(View, 'effective')
    def effective( self ):
        """ Dublin Core Date element - date resource becomes effective.
        """
        marker = []
        date = getattr( self, 'effective_date', marker )
        if date is marker:
            date = getattr( self, 'creation_date', None )
        return date is None and self.__FLOOR_DATE or date

    __CEILING_DATE = DateTime( 2500, 0 ) # never expires

    security.declareProtected(View, 'expires')
    def expires( self ):
        """ Dublin Core Date element - date resource expires.
        """
        date = getattr( self, 'expiration_date', None )
        return date is None and self.__CEILING_DATE or date

    security.declareProtected(View, 'modified')
    def modified( self ):
        """ Dublin Core Date element - date resource last modified.
        """
        date = self.modification_date
        if date is None:
            # Upgrade.
            date = self.bobobase_modification_time()
            self.modification_date = date
        return date

    security.declareProtected(View, 'getMetadataHeaders')
    def getMetadataHeaders( self ):
        """ Return RFC-822-style headers.
        """
        hdrlist = []
        hdrlist.append( ( 'Title', self.Title() ) )
        hdrlist.append( ( 'Subject', ', '.join( self.Subject() ) ) )
        hdrlist.append( ( 'Publisher', self.Publisher() ) )
        hdrlist.append( ( 'Description', self.Description() ) )
        hdrlist.append( ( 'Contributors', '; '.join( self.Contributors() ) ) )
        hdrlist.append( ( 'Effective_date', self.EffectiveDate() ) )
        hdrlist.append( ( 'Expiration_date', self.ExpirationDate() ) )
        hdrlist.append( ( 'Type', self.getPortalTypeName() or 'Unknown' ) )
        hdrlist.append( ( 'Format', self.Format() ) )
        hdrlist.append( ( 'Language', self.Language() ) )
        hdrlist.append( ( 'Rights', self.Rights() ) )
        return hdrlist

    #
    #  MutableDublinCore methods
    #
    security.declarePrivate( '_datify' )
    def _datify( self, attrib ):
        if attrib == 'None':
            attrib = None
        elif not isinstance( attrib, DateTime ):
            if attrib is not None:
                attrib = DateTime( attrib )
        return attrib

    security.declareProtected(ModifyPortalContent, 'setTitle')
    def setTitle( self, title ):
        """ Set Dublin Core Title element - resource name.
        """
        self.title = title

    security.declareProtected(ModifyPortalContent, 'setCreators')
    def setCreators(self, creators):
        """ Set Dublin Core Creator elements - resource authors.
        """
        self.creators = tuplize('creators', creators)

    security.declareProtected(ModifyPortalContent, 'setSubject')
    def setSubject( self, subject ):
        """ Set Dublin Core Subject element - resource keywords.
        """
        self.subject = tuplize( 'subject', subject )

    security.declareProtected(ModifyPortalContent, 'setDescription')
    def setDescription( self, description ):
        """ Set Dublin Core Description element - resource summary.
        """
        self.description = description

    security.declareProtected(ModifyPortalContent, 'setContributors')
    def setContributors( self, contributors ):
        """ Set Dublin Core Contributor elements - resource collaborators.
        """
        # XXX: fixme
        self.contributors = tuplize('contributors', contributors, semi_split)

    security.declareProtected(ModifyPortalContent, 'setEffectiveDate')
    def setEffectiveDate( self, effective_date ):
        """ Set Dublin Core Date element - date resource becomes effective.
        """
        self.effective_date = self._datify( effective_date )

    security.declareProtected(ModifyPortalContent, 'setExpirationDate')
    def setExpirationDate( self, expiration_date ):
        """ Set Dublin Core Date element - date resource expires.
        """
        self.expiration_date = self._datify( expiration_date )

    security.declareProtected(ModifyPortalContent, 'setFormat')
    def setFormat( self, format ):
        """ Set Dublin Core Format element - resource format.
        """
        self.format = format

    security.declareProtected(ModifyPortalContent, 'setLanguage')
    def setLanguage( self, language ):
        """ Set Dublin Core Language element - resource language.
        """
        self.language = language

    security.declareProtected(ModifyPortalContent, 'setRights')
    def setRights( self, rights ):
        """ Set Dublin Core Rights element - resource copyright.
        """
        self.rights = rights

    #
    #   Utility methods
    #
    security.declarePrivate('_editMetadata')
    def _editMetadata(self,
                      title=_marker,
                      subject=_marker,
                      description=_marker,
                      contributors=_marker,
                      effective_date=_marker,
                      expiration_date=_marker,
                      format=_marker,
                      language=_marker,
                      rights=_marker,
                      **kw):
        """ Update the editable metadata for this resource.
        """
        if title is not _marker:
            self.setTitle( title )
        if subject is not _marker:
            self.setSubject( subject )
        if description is not _marker:
            self.setDescription( description )
        if contributors is not _marker:
            self.setContributors( contributors )
        if effective_date is not _marker:
            self.setEffectiveDate( effective_date )
        if expiration_date is not _marker:
            self.setExpirationDate( expiration_date )
        if format is not _marker:
            self.setFormat( format )
        if language is not _marker:
            self.setLanguage( language )
        if rights is not _marker:
            self.setRights( rights )

    #
    #   ZMI methods
    #
    security.declareProtected(ModifyPortalContent, 'manage_metadata')
    manage_metadata = DTMLFile( 'zmi_metadata', _dtmldir )

    security.declareProtected(ModifyPortalContent, 'manage_editMetadata')
    def manage_editMetadata( self
                           , title
                           , subject
                           , description
                           , contributors
                           , effective_date
                           , expiration_date
                           , format
                           , language
                           , rights
                           , REQUEST
                           ):
        """ Update metadata from the ZMI.
        """
        self._editMetadata( title, subject, description, contributors
                          , effective_date, expiration_date
                          , format, language, rights
                          )
        REQUEST[ 'RESPONSE' ].redirect( self.absolute_url()
                                + '/manage_metadata'
                                + '?manage_tabs_message=Metadata+updated.' )

    security.declareProtected(ModifyPortalContent, 'editMetadata')
    def editMetadata(self
                   , title=''
                   , subject=()
                   , description=''
                   , contributors=()
                   , effective_date=None
                   , expiration_date=None
                   , format='text/html'
                   , language='en-US'
                   , rights=''
                    ):
        """
        Need to add check for webDAV locked resource for TTW methods.
        """
        # as per bug #69, we cant assume they use the webdav
        # locking interface, and fail gracefully if they dont
        if hasattr(self, 'failIfLocked'):
            self.failIfLocked()

        self._editMetadata(title=title
                     , subject=subject
                     , description=description
                     , contributors=contributors
                     , effective_date=effective_date
                     , expiration_date=expiration_date
                     , format=format
                     , language=language
                     , rights=rights
                     )
        self.reindexObject()

InitializeClass(DefaultDublinCoreImpl)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""CMFDefault formlib tests.
"""

import doctest
import unittest
from Testing import ZopeTestCase

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.testing import FunctionalLayer
from zope.component.factory import Factory
from zope.interface import implements
from zope.interface import Interface
from zope.schema import TextLine

CONTENT_ZCML = """\
<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:five="http://namespaces.zope.org/five">

  <five:registerClass
      class="Products.CMFDefault.formlib.tests.Foo"
      meta_type="Foo Type"
      permission="cmf.AddPortalContent"
      />

  <utility
      component="Products.CMFDefault.formlib.tests.FooFactory"
      name="test.foo"
      />

</configure>
"""

TYPES_XML = """\
<?xml version="1.0"?>
<object name="portal_types">
 <object name="MyFoo" meta_type="Factory-based Type Information"/>
</object>
"""

MYFOO_XML = """\
<?xml version="1.0"?>
<object name="MyFoo">
 <property name="content_meta_type">Foo Type</property>
 <property name="factory">test.foo</property>
 <property name="add_view_expr">string:${folder_url}/++add++MyFoo</property>
 <action title="View" action_id="view" category="object" url_expr=""/>
</object>
"""


class IFoo(Interface):

    bar = TextLine(title=u'Bar')
    baz = TextLine(title=u'Baz')


class Foo(PortalContent, DefaultDublinCoreImpl):

    implements(IFoo)

    def __init__(self, id, bar='', baz=''):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.bar = bar
        self.baz = baz

FooFactory = Factory(Foo)


def test_suite():
    suite = unittest.TestSuite()
    s = ZopeTestCase.FunctionalDocFileSuite('form.txt')
    s.layer = FunctionalLayer
    suite.addTest(s)
    suite.addTest(doctest.DocFileSuite('schema.txt',
                                    optionflags=doctest.NORMALIZE_WHITESPACE))
    suite.addTest(doctest.DocFileSuite('widgets.txt',
                                    optionflags=doctest.NORMALIZE_WHITESPACE |
                                                doctest.ELLIPSIS))
    return suite



##parameters=
##title=clear browser cookie
##
REQUEST=context.REQUEST
REQUEST.RESPONSE.expireCookie('folderfilter', path='/')
REQUEST.RESPONSE.redirect( context.absolute_url() + '/folder_contents?portal_status_message=Filter+cleared.')



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for adding members.
"""

import unittest
from Testing import ZopeTestCase

from zope.site.hooks import setSite

from Products.CMFDefault.testing import FunctionalLayer


class MembershipTests(ZopeTestCase.FunctionalTestCase):

    layer = FunctionalLayer

    def afterSetUp(self):
        setSite(self.app.site)

    def test_join( self ):
        site = self.app.site
        member_id = 'test_user'

        site.portal_registration.addMember( member_id
                                          , 'zzyyzz'
                                          , properties={ 'username': member_id
                                                       , 'email' : 'foo@bar.com'
                                                       }
                                          )
        u = site.acl_users.getUser(member_id)
        self.failUnless(u)

    def test_join_memberproperties(self):
        # Make sure the member data wrapper carries correct properties
        # after joining
        site = self.app.site
        member_id = 'test_user'

        site.portal_registration.addMember( member_id
                                          , 'zzyyzz'
                                          , properties={ 'username': member_id
                                                       , 'email' : 'foo@bar.com'
                                                       }
                                          )

        m = site.portal_membership.getMemberById('test_user')
        self.assertEqual(m.getProperty('email'), 'foo@bar.com')
        self.assertEqual(m.getMemberId(), member_id)
        self.assertEqual(m.getRoles(), ('Member', 'Authenticated'))

    def test_join_without_email( self ):
        site = self.app.site

        self.assertRaises(ValueError,
                          site.portal_registration.addMember,
                          'test_user',
                          'zzyyzz',
                          properties={'username':'test_user', 'email': ''}
                          )

    def test_join_with_variable_id_policies( self ):
        site = self.app.site
        member_id = 'test.user'

        # Test with the default policy: Names with "." should fail
        self.assertRaises(ValueError,
                          site.portal_registration.addMember,
                          member_id,
                          'zzyyzz',
                          properties={ 'username':'Test User'
                                     , 'email': 'foo@bar.com'
                                     }
                          )

        # Now change the policy to allow "."
        new_pattern = "^[A-Za-z][A-Za-z0-9_\.]*$"
        site.portal_registration.manage_editIDPattern(new_pattern)
        site.portal_registration.addMember( member_id
                                          , 'zzyyzz'
                                          , properties={ 'username': 'TestUser2'
                                                       , 'email' : 'foo@bar.com'
                                                       }
                                          )
        u = site.acl_users.getUser(member_id)
        self.failUnless(u)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(MembershipTests),
        ))



##parameters=
##
from ZTUtils import Batch
from ZTUtils import LazyFilter
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode

stool = getUtilityByInterfaceName('Products.CMFCore.interfaces.ISyndicationTool')
utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')


if not stool.isSyndicationAllowed(context):
    context.REQUEST.RESPONSE.redirect(context.absolute_url() +
             '/rssDisabled?portal_status_message=Syndication+is+Disabled')
    return

options = {}

syndication_info = stool.getSyndicationInfo(context)
converter = {'hourly': 1, 'daily': 24, 'weekly': 7 * 24, 'monthly': 30 * 24,
             'yearly': 365 * 24}
ttl = 60 * (syndication_info['frequency'] *
            converter[syndication_info['period']])

syndication_info.update({'description': context.Description(),
                         'title': context.Title(),
                         'url': context.absolute_url(),
                         'ttl': ttl,
                         'portal_url': utool()}
                        )
syndication_info['base'] = syndication_info['base'].rfc822()

options['channel_info'] = syndication_info

key, reverse = context.getDefaultSorting()
items = stool.getSyndicatableContent(context)
items = sequence.sort( items, ((key, 'cmp', reverse and 'desc' or 'asc'),) )
items = LazyFilter(items, skip='View')
b_size = stool.getMaxItems(context)
batch_obj = Batch(items, b_size, 0, orphan=0)
items = []
for item in batch_obj:
    items.append( { 'date': item.modified().rfc822(),
                    'description': item.Description(),
                    'listCreators': item.listCreators(),
                    'listSubjects': item.Subject(),
                    'publisher': item.Publisher(),
                    'rights': item.Rights(),
                    'title': item.Title(),
                    'url': item.absolute_url(),
                    'uid': None } )
options['listItemInfos'] = tuple(items)

return context.RSS_template(**decode(options, script))



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Upgrade steps to CMFDefault 2.2.
"""
import logging
from urllib import quote

from AccessControl.Permissions import access_contents_information
from AccessControl.Permissions import view
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from zope.component.interfaces import ComponentLookupError

from Products.CMFCore.interfaces import IWorkflowDefinition
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.MetadataTool import MetadataSchema
from Products.CMFDefault.MetadataTool import _DCMI_ELEMENT_SPECS

_KNOWN_IMPORT_STEPS = (
    'actions',
    'caching_policy_mgr',
    'catalog',
    'componentregistry',
    'content_type_registry',
    'cookie_authentication',
    'mailhost',
    'properties',
    'rolemap',
    'skins',
    'toolset',
    'typeinfo',
    'various',
    'workflow',
    )

_KNOWN_EXPORT_STEPS = (
    'actions',
    'caching_policy_mgr',
    'catalog',
    'componentregistry',
    'content_type_registry',
    'cookieauth',
    'mailhost',
    'properties',
    'rolemap',
    'skins',
    'step_registries',
    'toolset',
    'typeinfo',
    'workflows',
    )

def check_setup_tool(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    registry = tool.getImportStepRegistry()
    steps = registry.listSteps()
    for step in _KNOWN_IMPORT_STEPS:
        if step in steps:
            return True
    registry = tool.getExportStepRegistry()
    steps = registry.listSteps()
    for step in _KNOWN_EXPORT_STEPS:
        if step in steps:
            return True
    return False

def upgrade_setup_tool(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    registry = tool.getImportStepRegistry()
    steps = registry.listSteps()
    for step in _KNOWN_IMPORT_STEPS:
        if step in steps:
            registry.unregisterStep(step)
            tool._p_changed = True
            logger.info("Import step '%s' locally unregistered." % step)
    registry = tool.getExportStepRegistry()
    steps = registry.listSteps()
    for step in _KNOWN_EXPORT_STEPS:
        if step in steps:
            registry.unregisterStep(step)
            tool._p_changed = True
            logger.info("Export step '%s' locally unregistered." % step)

def check_root_site_manager(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    portal = aq_parent(aq_inner(tool))
    try:
        components = portal.getSiteManager()
    except ComponentLookupError:
        return True
    return components.__name__ != '++etc++site'

def upgrade_root_site_manager(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    portal = aq_parent(aq_inner(tool))
    try:
        components = portal.getSiteManager()
    except ComponentLookupError:
        logger.warning("Site manager missing.")
        return
    components.__name__ = '++etc++site'
    logger.info("Site manager name changed to '++etc++site'.")

def check_root_properties(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    portal = aq_parent(aq_inner(tool))
    return not portal.hasProperty('enable_actionicons')

def upgrade_root_properties(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    portal = aq_parent(aq_inner(tool))
    portal.manage_addProperty('enable_actionicons', False, 'boolean')
    logger.info("'enable_actionicons' property added.")

def check_type_properties(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    ttool = getToolByName(tool, 'portal_types')
    for ti in ttool.listTypeInfo():
        content_icon = getattr(ti, 'content_icon', None)
        if content_icon is not None:
            return True
        if not ti.getProperty('add_view_expr') and \
                ti.getProperty('content_meta_type') != 'Discussion Item':
            return True
    return False

def upgrade_type_properties(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    ttool = getToolByName(tool, 'portal_types')
    for ti in ttool.listTypeInfo():
        changed = False
        content_icon = getattr(ti, 'content_icon', None)
        if content_icon is not None:
            del ti.content_icon
            if content_icon and not ti.getProperty('icon_expr'):
                icon_expr = 'string:${portal_url}/%s' % content_icon
                ti._updateProperty('icon_expr', icon_expr)
            changed = True
        if not ti.getProperty('add_view_expr') and \
                ti.getProperty('content_meta_type') != 'Discussion Item':
            ti._updateProperty('add_view_expr',
                               'string:${folder_url}/++add++%s'
                               % quote(ti.getId()))
            changed = True
        if changed:
            logger.info("TypeInfo '%s' changed." % ti.getId())

_ACTION_ICONS = {'download': 'download_icon.png',
                 'edit': 'edit_icon.png',
                 'folderContents': 'folder_icon.png',
                 'localroles': 'localroles_icon.png',
                 'metadata': 'metadata_icon.png',
                 'view': 'preview_icon.png',
                 'publish': 'approve_icon.png',
                 'reject': 'reject_icon.png',
                 'retract': 'retract_icon.png',
                 'submit': 'submit_icon.png',
                 'reviewer_queue': 'worklist_icon.png',
                 'login': 'login_icon.png',
                 'join': 'join_icon.png',
                 'preferences': 'preferences_icon.png',
                 'logout': 'logout_icon.png',
                 'addFavorite': 'addfavorite_icon.png',
                 'mystuff': 'user_icon.png',
                 'favorites': 'favorite_icon.png',
                 'reply': 'reply_icon.png',
                 'syndication': 'syndication_icon.png',
                 'interfaces': 'interfaces_icon.png',
                 'folderContents': 'folder_icon.png',
                 'manage_members': 'members_icon.png',
                 'undo': 'undo_icon.png',
                 'configPortal': 'tool_icon.png',
                 }

def check_type_actions(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    ttool = getToolByName(tool, 'portal_types')
    for ti in ttool.listTypeInfo():
        for ai in ti.listActions():
            if not ai.getIconExpression() and ai.getId() in _ACTION_ICONS:
                return True
    return False

def upgrade_type_actions(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    ttool = getToolByName(tool, 'portal_types')
    for ti in ttool.listTypeInfo():
        changed = False
        for ai in ti.listActions():
            if not ai.getIconExpression() and ai.getId() in _ACTION_ICONS:
                ai.setIconExpression('string:${portal_url}/%s'
                                     % _ACTION_ICONS[ai.getId()])
                changed = True
        if changed:
            logger.info("TypeInfo '%s' changed." % ti.getId())

def check_workflow_definitions(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    wtool = getToolByName(tool, 'portal_workflow')
    for obj in wtool.objectValues():
        if IWorkflowDefinition.providedBy(obj):
            for t in obj.transitions.values():
                if not t.actbox_icon and t.getId() in _ACTION_ICONS:
                    return True
            for w in obj.worklists.values():
                if not w.actbox_icon and w.getId() in _ACTION_ICONS:
                    return True
    return False

def upgrade_workflow_definitions(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    wtool = getToolByName(tool, 'portal_workflow')
    for wf in wtool.objectValues():
        changed = False
        if IWorkflowDefinition.providedBy(wf):
            for t in wf.transitions.values():
                if not t.actbox_icon and t.getId() in _ACTION_ICONS:
                    icon = _ACTION_ICONS[t.getId()]
                    t.actbox_icon = '%(portal_url)s/' + icon
                    changed = True
            for w in wf.worklists.values():
                if not w.actbox_icon and w.getId() in _ACTION_ICONS:
                    icon = _ACTION_ICONS[w.getId()]
                    w.actbox_icon = '%(portal_url)s/' + icon
                    changed = True
        if changed:
            logger.info("WorkflowDefinition '%s' changed." % wf.getId())

def check_action_properties(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    atool = getToolByName(tool, 'portal_actions')
    for category in atool.objectValues():
        for action in category.listActions():
            if not action.icon_expr and action.getId() in _ACTION_ICONS:
                return True
    return False

def upgrade_action_properties(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    atool = getToolByName(tool, 'portal_actions')
    for category in atool.objectValues():
        for action in category.listActions():
            if not action.icon_expr and action.getId() in _ACTION_ICONS:
                icon = _ACTION_ICONS[action.getId()]
                icon = 'string:${portal_url}/%s' % icon
                action._setPropValue('icon_expr', icon)
                logger.info("Action '%s' changed." % action.getId())

def check_catalog_columns(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    ctool = getToolByName(tool, 'portal_catalog')
    columns = ctool.schema()
    if 'getIcon' in columns:
        return True
    if 'getIconURL' not in columns:
        return True
    return False

def upgrade_catalog_columns(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    ctool = getToolByName(tool, 'portal_catalog')
    columns = ctool.schema()
    if 'getIcon' in columns:
        ctool.delColumn('getIcon')
        logger.info("Catalog column 'getIcon' deleted.")
    if 'getIconURL' not in columns:
        ctool.addColumn('getIconURL')
        logger.info("Catalog column 'getIconURL' added.")

def check_dcmi_metadata(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    metadata_tool = getToolByName(tool, 'portal_metadata')
    return getattr(aq_base(metadata_tool), 'DCMI', None) is None

def upgrade_dcmi_metadata(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    metadata_tool = getToolByName(tool, 'portal_metadata')
    if getattr(aq_base(metadata_tool), 'DCMI', None) is None:
        if getattr(aq_base(metadata_tool), '_DCMI', None) is None:
            metadata_tool.DCMI = MetadataSchema('DCMI', _DCMI_ELEMENT_SPECS)
        else:
            metadata_tool.DCMI = metadata_tool._DCMI
            del metadata_tool._DCMI
    logger.info('Dublin Core metadata definition updated.')


_SINGLESTATE_WF_ID = 'singlestate_workflow'

def check_singlestate_workflow(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    wf_tool = getToolByName(tool, 'portal_workflow')
    return wf_tool.getWorkflowById(_SINGLESTATE_WF_ID) is None

def add_singlestate_workflow(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    wf_tool = getToolByName(tool, 'portal_workflow')
    if wf_tool.getWorkflowById(_SINGLESTATE_WF_ID) is None:
        from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
        wf = DCWorkflowDefinition(_SINGLESTATE_WF_ID)
        wf.title = 'Single-state workflow'
        wf.initial_state = 'published'
        wf.state_var = 'review_state'
        wf.manager_bypass = False
        wf.permissions = ( access_contents_information
                         , ModifyPortalContent
                         , view
                         )
        wf.states.addState('published')
        public = wf.states.published
        public.title = 'Public'
        public.setPermission( access_contents_information
                            , True
                            , ('Anonymous', 'Manager')
                            )
        public.setPermission(ModifyPortalContent, False, ('Manager', 'Owner'))
        public.setPermission(view, True, ('Anonymous', 'Manager'))
        wf_tool._setObject(_SINGLESTATE_WF_ID, wf)

def check_discussionitem_workflow(tool):
    """2.1.x to 2.2.0 upgrade step checker
    """
    wf_tool = getToolByName(tool, 'portal_workflow')
    discussion_overrides = [x for x in wf_tool.listChainOverrides()
                                               if x[0] == 'Discussion Item']

    if not discussion_overrides: 
        return True

    # Only apply if Discussion Item has an empty workflow chain
    if not discussion_overrides[0][1]:
        return True

    return False

def upgrade_discussionitem_workflow(tool):
    """2.1.x to 2.2.0 upgrade step handler
    """
    wf_tool = getToolByName(tool, 'portal_workflow')
    wf_tool.setChainForPortalTypes(('Discussion Item',), _SINGLESTATE_WF_ID,
                                   verify=False)



##parameters=b_start=0
##
from ZTUtils import Batch
from ZTUtils import LazyFilter
from Products.CMFDefault.utils import decode

options = {}

options['has_local'] = 'local_pt' in context.objectIds()

key, reverse = context.getDefaultSorting()
items = context.contentValues()
items = sequence.sort( items, ((key, 'cmp', reverse and 'desc' or 'asc'),) )
items = LazyFilter(items, skip='View')
batch_obj = Batch(items, 25, b_start, orphan=0)
listItemInfos = context.getBatchItemInfos(batch_obj)
target = context.getActionInfo('object/view')['url']
navigation = context.getBatchNavigation(batch_obj, target)
options['batch'] = { 'listItemInfos': listItemInfos,
                     'navigation': navigation }

return context.index_html_template(**decode(options, script))



##############################################################################
#
# Copyright (c) 2008 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Upgrade steps to CMFDefault 2.1.

$Id: to21.py 111306 2010-04-23 16:01:40Z yuppie $
"""
import logging

from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from five.localsitemanager import find_next_sitemanager
from five.localsitemanager.registry import FiveVerifyingAdapterLookup
from five.localsitemanager.registry import PersistentComponents
from zope.component import getMultiAdapter
from zope.component.globalregistry import base
from zope.component.interfaces import ComponentLookupError
from zope.dottedname.resolve import resolve
from zope.site.hooks import setSite

from Products.CMFCore.DirectoryView import _dirreg
from Products.CMFCore.DirectoryView import _generateKey
from Products.CMFCore.interfaces import IDirectoryView
from Products.CMFCore.utils import getToolByName
from Products.GenericSetup.context import SetupEnviron
from Products.GenericSetup.interfaces import IBody

_COMPONENTS_XML = """\
<?xml version="1.0"?>
<componentregistry>
 <adapters/>
 <utilities>
  <utility interface="Products.CMFCore.interfaces.IDiscussionTool"
     object="portal_discussion"/>
  <utility interface="Products.CMFCore.interfaces.IMetadataTool"
     object="portal_metadata"/>
  <utility interface="Products.CMFCore.interfaces.IPropertiesTool"
     object="portal_properties"/>
  <utility interface="Products.CMFCore.interfaces.ISiteRoot" object=""/>
  <utility interface="Products.CMFCore.interfaces.ISyndicationTool"
     object="portal_syndication"/>
  <utility interface="Products.CMFCore.interfaces.IUndoTool"
     object="portal_undo"/>
  <utility interface="Products.GenericSetup.interfaces.ISetupTool"
     object="portal_setup"/>
  <utility interface="Products.MailHost.interfaces.IMailHost"
     object="MailHost"/>
 </utilities>
</componentregistry>
"""

_ACTIONS_XML = """\
<?xml version="1.0"?>
<object name="portal_actions" meta_type="CMF Actions Tool"
   xmlns:i18n="http://xml.zope.org/namespaces/i18n">
 <object name="object" meta_type="CMF Action Category">
  <object name="interfaces" meta_type="CMF Action" i18n:domain="cmf_default">
   <property name="title" i18n:translate="">Interfaces</property>
   <property name="description"
      i18n:translate="">Assign marker interfaces</property>
   <property name="url_expr">string:${object_url}/edit-markers.html</property>
   <property name="link_target"></property>
   <property
      name="icon_expr">string:${portal_url}/interfaces_icon.png</property>
   <property name="available_expr"></property>
   <property name="permissions">
    <element value="Manage portal"/>
   </property>
   <property name="visible">True</property>
  </object>
 </object>
</object>
"""

_BAD_UTILITIES = [
         'Products.CMFCalendar.interfaces.ICalendarTool',
         'Products.CMFCore.interfaces.IActionsTool',
         'Products.CMFCore.interfaces.ICatalogTool',
         'Products.CMFCore.interfaces.IContentTypeRegistry',
         'Products.CMFCore.interfaces.ISkinsTool',
         'Products.CMFCore.interfaces.ITypesTool',
         'Products.CMFCore.interfaces.IURLTool',
         'Products.CMFCore.interfaces.IConfigurableWorkflowTool',
         'Products.CMFCore.interfaces.IMembershipTool',
         'Products.CMFCore.interfaces.IRegistrationTool',
         ]

_TOOL_UTILITIES = (
    ('portal_uidgenerator', 'Products.CMFUid.interfaces.IUniqueIdGenerator'),
    ('portal_uidannotation', 'Products.CMFUid.interfaces.IUniqueIdAnnotationManagement'),
    ('portal_uidhandler', 'Products.CMFUid.interfaces.IUniqueIdHandler'),
    ('portal_actionicons', 'Products.CMFActionIcons.interfaces.IActionIconsTool'),
)

def check_root_site_manager(tool):
    """2.0.x to 2.1.0 upgrade step checker
    """
    portal = aq_parent(aq_inner(tool))
    try:
        # We have to call setSite to make sure we have a site with a proper
        # acquisition context.
        setSite(portal)
        sm = portal.getSiteManager()
        if sm.utilities.LookupClass != FiveVerifyingAdapterLookup:
            return True
    except ComponentLookupError:
        return True

    for tool_interface in _BAD_UTILITIES:
        try:
            iface = resolve(tool_interface)
        except ImportError:
            continue

        if sm.queryUtility(iface) is not None:
            return True

    for tool_id, tool_interface in _TOOL_UTILITIES:
        tool_obj = getToolByName(portal, tool_id, default=None)
        try:
            iface = resolve(tool_interface)
        except ImportError:
            continue

        if tool_obj is not None and sm.queryUtility(iface) is None:
            return True

    return False

def upgrade_root_site_manager(tool):
    """2.0.x to 2.1.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    portal = aq_parent(aq_inner(tool))
    try:
        setSite(portal)
        sm = portal.getSiteManager()
        if sm.utilities.LookupClass != FiveVerifyingAdapterLookup:
            sm.__parent__ = aq_base(portal)
            sm.utilities.LookupClass = FiveVerifyingAdapterLookup
            sm.utilities._createLookup()
            sm.utilities.__parent__ = sm
            logger.info('LookupClass replaced.')
        else:
            for tool_interface in _BAD_UTILITIES:
                try:
                    iface = resolve(tool_interface)
                except ImportError:
                    continue

                if sm.queryUtility(iface) is not None:
                    sm.unregisterUtility(provided=iface)
                    logger.info('Unregistered utility for %s' % tool_interface)

            for tool_id, tool_interface in _TOOL_UTILITIES:
                tool_obj = getToolByName(portal, tool_id, default=None)
                try:
                    iface = resolve(tool_interface)
                except ImportError:
                    continue

                if tool_obj is not None and sm.queryUtility(iface) is None:
                    sm.registerUtility(tool_obj, iface)
                    logger.info('Registered %s for interface %s' % (
                                                      tool_id, tool_interface))
            return
    except ComponentLookupError:
        next = find_next_sitemanager(portal)
        if next is None:
            next = base
        name = '/'.join(portal.getPhysicalPath())
        sm = PersistentComponents(name, (next,))
        sm.__parent__ = aq_base(portal)
        portal.setSiteManager(sm)
        logger.info("Site manager '%s' added." % name)
    getMultiAdapter((sm, SetupEnviron()), IBody).body = _COMPONENTS_XML
    logger.info('Utility registrations added.')

def check_root_properties(tool):
    """2.0.x to 2.1.0 upgrade step checker
    """
    portal = aq_parent(aq_inner(tool))
    return not portal.hasProperty('email_charset')

def upgrade_root_properties(tool):
    """2.0.x to 2.1.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    portal = aq_parent(aq_inner(tool))
    portal.manage_addProperty('email_charset', 'iso-8859-1', 'string')
    prop_map = list(portal._properties)
    for i in range(len(prop_map)):
        if prop_map[i]['id'] == 'default_charset':
            email_charset_info = prop_map.pop(-1)
            prop_map.insert(i+1, email_charset_info)
            portal._properties = tuple(prop_map)
            break
    logger.info("'email_charset' property added.")

_FACTORIES = {
    'CMFCore-manage_addPortalFolder': 'cmf.folder',
    'CMFCore-manage_addCMFBTreeFolder': 'cmf.folder.btree',
    'CMFDefault-addDocument': 'cmf.document',
    'CMFDefault-addFavorite': 'cmf.favorite',
    'CMFDefault-addFile': 'cmf.file',
    'CMFDefault-addImage': 'cmf.image',
    'CMFDefault-addLink': 'cmf.link',
    'CMFDefault-addNewsItem': 'cmf.newsitem'}

def check_type_properties(tool):
    """2.0.x to 2.1.0 upgrade step checker
    """
    ttool = getToolByName(tool, 'portal_types')
    for ti in ttool.listTypeInfo():
        key = '%s-%s' % (ti.getProperty('product'), ti.getProperty('factory'))
        if key in _FACTORIES:
            return True
    return False

def upgrade_type_properties(tool):
    """2.0.x to 2.1.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    ttool = getToolByName(tool, 'portal_types')
    for ti in ttool.listTypeInfo():
        key = '%s-%s' % (ti.getProperty('product'), ti.getProperty('factory'))
        if key in _FACTORIES:
            ti._updateProperty('product', '')
            ti._updateProperty('factory', _FACTORIES[key])
            logger.info("TypeInfo '%s' changed." % ti.getId())

def check_actions_tool(tool):
    """2.0.x to 2.1.0 upgrade step checker
    """
    atool = getToolByName(tool, 'portal_actions')
    try:
        atool.object.interfaces
    except AttributeError:
        return True
    return False

def upgrade_actions_tool(tool):
    """2.0.x to 2.1.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    atool = getToolByName(tool, 'portal_actions')
    environ = SetupEnviron()
    environ._should_purge = False
    getMultiAdapter((atool, environ), IBody).body = _ACTIONS_XML
    logger.info("'interfaces' action added.")

def check_skins_tool(tool):
    """2.0.x to 2.1.0 upgrade step checker
    """
    stool = getToolByName(tool, 'portal_skins')
    for obj in stool.objectValues():
        if IDirectoryView.providedBy(obj):
            dirpath = obj.getDirPath()
            if dirpath is None:
                continue
            if dirpath not in _dirreg.listDirectories():
                return True
    return False

def _getCurrentKeyFormat(reg_key):
    dirpath = reg_key.replace('\\', '/')
    if dirpath.startswith('Products/'):
        dirpath = dirpath[9:]
    product = ['Products']
    dirparts = dirpath.split('/')
    while dirparts:
        product.append(dirparts[0])
        dirparts = dirparts[1:]
        possible_key = _generateKey('.'.join(product), '/'.join(dirparts))
        if possible_key in _dirreg._directories:
            return possible_key
    return reg_key

def upgrade_skins_tool(tool):
    """2.0.x to 2.1.0 upgrade step handler
    """
    logger = logging.getLogger('GenericSetup.upgrade')
    stool = getToolByName(tool, 'portal_skins')
    for obj in stool.objectValues():
        if IDirectoryView.providedBy(obj):
            dirpath = obj.getDirPath()
            if dirpath is None:
                continue
            if dirpath not in _dirreg.listDirectories():
                obj._dirpath = _getCurrentKeyFormat(dirpath)
                logger.info("DirectoryView '%s' changed." % obj.getId())



##parameters=member=None, password='secret'
##
from Products.CMFCore.utils import getUtilityByInterfaceName
from Products.CMFDefault.utils import decode
from Products.CMFDefault.utils import makeEmail
from Products.CMFDefault.utils import Message as _

atool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IActionsTool')
ptool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IPropertiesTool')
utool = getUtilityByInterfaceName('Products.CMFCore.interfaces.IURLTool')
portal_url = utool()


options = {}
options['password'] = password

headers = {}
headers['Subject'] = _(u'${portal_title}: Membership reminder',
                      mapping={'portal_title': decode(ptool.title(), script)})
headers['From'] = '%s <%s>' % (ptool.getProperty('email_from_name'),
                               ptool.getProperty('email_from_address'))
headers['To'] = '<%s>' % (member and member.getProperty('email') or
                          'foo@example.org')

mtext = context.password_email_template(**decode(options, script))
return makeEmail(mtext, script, headers)



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for NewsItem module.
"""

import unittest
import Testing

from zope.interface.verify import verifyClass

from Products.CMFCore.testing import ConformsToContent
from Products.CMFCore.tests.base.content import BASIC_HTML
from Products.CMFCore.tests.base.content import BASIC_STRUCTUREDTEXT
from Products.CMFCore.tests.base.content import DOCTYPE
from Products.CMFCore.tests.base.content import ENTITY_IN_TITLE
from Products.CMFCore.tests.base.testcase import TransactionalTest


class NewsItemTests(ConformsToContent, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.NewsItem import NewsItem

        return NewsItem

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_interfaces(self):
        from Products.CMFDefault.interfaces import IDocument
        from Products.CMFDefault.interfaces import IMutableNewsItem
        from Products.CMFDefault.interfaces import INewsItem

        verifyClass(IDocument, self._getTargetClass())
        # XXX: different signature
        # verifyClass(IMutableDocument, self._getTargetClass())
        verifyClass(IMutableNewsItem, self._getTargetClass())
        verifyClass(INewsItem, self._getTargetClass())

    def test_Empty_html(self):
        d = self._makeOne('empty', text_format='html')

        self.assertEqual(d.Title(), '')
        self.assertEqual(d.Description(), '')
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.text_format, 'html')
        self.assertEqual(d.text, '')

    def test_Empty_stx(self):
        d = self._makeOne('foo', text_format='structured-text')

        self.assertEqual(d.Title(), '')
        self.assertEqual(d.Description(), '')
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.text_format, 'structured-text')
        self.assertEqual(d.text, '')

    def test_Init_with_stx(self):
        d = self._makeOne('foo', text_format='structured-text',
                          title='Foodoc')

        self.assertEqual(d.Title(), 'Foodoc')
        self.assertEqual(d.Description(), '')
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.text_format, 'structured-text')
        self.assertEqual(d.text, '')

    def test_default_format(self):
        d = self._makeOne('foo', text='')

        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.text_format, 'structured-text')


class NewsItemPUTTests(TransactionalTest):

    def _getTargetClass(self):
        from Products.CMFDefault.NewsItem import NewsItem

        return NewsItem

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_Init(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo', text='')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), 'My Document')
        self.assertEqual(d.Description(), 'A document by me')
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.text_format, 'structured-text')
        self.assertEqual(len(d.Contributors()), 3)
        self.assertTrue(d.cooked_text.find('<p>') >= 0)

    def test_PUT_basic_html(self):
        self.REQUEST['BODY'] = BASIC_HTML
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), 'Title in tag')
        self.assertEqual(d.Description(), 'Describe me')
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.text_format, 'html')
        self.assertEqual(d.text.find('</body>'), -1)
        self.assertEqual(len(d.Contributors()), 3)

    def test_PUT_uppered_html(self):
        self.REQUEST['BODY'] = BASIC_HTML.upper()
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), 'TITLE IN TAG')
        self.assertEqual(d.Description(), 'DESCRIBE ME')
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.text_format, 'html')
        self.assertEqual(d.text.find('</BODY'), -1)
        self.assertEqual(len(d.Contributors()), 3)

    def test_PUT_entity_in_title(self):
        self.REQUEST['BODY'] = ENTITY_IN_TITLE
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), '&Auuml;rger')

    def test_PUT_html_with_doctype(self):
        self.REQUEST['BODY'] = '%s\n%s' % (DOCTYPE, BASIC_HTML)
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Description(), 'Describe me')

    def test_PUT_html_without_newlines(self):
        self.REQUEST['BODY'] = ''.join(BASIC_HTML.split('\n'))
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), 'Title in tag')
        self.assertEqual(d.Description(), 'Describe me')
        self.assertEqual(d.Format(), 'text/html')
        self.assertEqual(d.text_format, 'html')
        self.assertEqual(d.text.find('</body>'), -1)
        self.assertEqual(len(d.Contributors()), 3)

    def test_PUT_structured_text(self):
        self.REQUEST['BODY'] = BASIC_STRUCTUREDTEXT
        d = self._makeOne('foo')
        d.PUT(self.REQUEST, self.RESPONSE)

        self.assertEqual(d.Title(), 'My Document')
        self.assertEqual(d.Description(), 'A document by me')
        self.assertEqual(d.Format(), 'text/plain')
        self.assertEqual(d.text_format, 'structured-text')
        self.assertEqual(len(d.Contributors()), 3)
        self.assertTrue(d.cooked_text.find('<p>') >= 0)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(NewsItemTests),
        unittest.makeSuite(NewsItemPUTTests),
        ))



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Default implementation of CMFCore.

$Id: __init__.py 110420 2010-04-01 17:12:56Z tseaver $
"""

def initialize(context):
    from Products.CMFCore.utils import ToolInit
    from Products.CMFCore.utils import ContentInit
    from Products.CMFCore.utils import registerIcon
    from Products.CMFDefault.permissions import AddPortalContent

    import DefaultWorkflow
    import DiscussionTool
    import Document
    import factory
    import Favorite
    import File
    import Image
    import Link
    import MembershipTool
    import MetadataTool
    import NewsItem
    import Portal
    import PropertiesTool
    import RegistrationTool
    import SkinnedFolder
    import SyndicationTool

    # Make sure security is initialized
    import DiscussionItem
    import DublinCore
    import utils

    contentConstructors = ( Document.addDocument
                        , File.addFile
                        , Image.addImage
                        , Link.addLink
                        , Favorite.addFavorite
                        , NewsItem.addNewsItem
                        , SkinnedFolder.addSkinnedFolder
                        )

    tools = ( DiscussionTool.DiscussionTool
            , MembershipTool.MembershipTool
            , RegistrationTool.RegistrationTool
            , PropertiesTool.PropertiesTool
            , MetadataTool.MetadataTool
            , SyndicationTool.SyndicationTool
            )

    # Register the ID to interface data for those items that don't have their 
    # own module
    from Products.CMFCore.utils import registerToolInterface
    from Products.GenericSetup.interfaces import ISetupTool
    registerToolInterface('portal_setup', ISetupTool)
    from Products.MailHost.interfaces import IMailHost
    registerToolInterface('MailHost', IMailHost)

    ToolInit( 'CMF Default Tool'
            , tools=tools
            , icon='tool.gif'
            ).initialize( context )

    # BBB: register oldstyle constructors
    ContentInit( 'CMF Default Content'
               , content_types=()
               , permission=AddPortalContent
               , extra_constructors=contentConstructors
               , visibility=None
               ).initialize( context )

    context.registerClass( Portal.CMFSite
                         , constructors=(factory.addConfiguredSiteForm,
                                         factory.addConfiguredSite)
                         , icon='images/portal.gif'
                         )

    registerIcon( DefaultWorkflow.DefaultWorkflowDefinition
                , 'images/workflow.gif'
                , globals()
                )

    # make registerHelp work with 2 directories
    help = context.getProductHelp()
    lastRegistered = help.lastRegistered
    context.registerHelp(directory='help', clear=1)
    context.registerHelp(directory='interfaces', clear=1)
    if help.lastRegistered != lastRegistered:
        help.lastRegistered = None
        context.registerHelp(directory='help', clear=1)
        help.lastRegistered = None
        context.registerHelp(directory='interfaces', clear=0)
    context.registerHelpTitle('CMF Default Help')



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for Discussions.
"""

import unittest
import Testing

from AccessControl.SecurityManagement import newSecurityManager
from zope.component import getSiteManager
from zope.interface.verify import verifyClass

from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFCore.interfaces import ICatalogTool
from Products.CMFCore.interfaces import IDiscussionTool
from Products.CMFCore.interfaces import ITypesTool
from Products.CMFCore.testing import EventZCMLLayer
from Products.CMFCore.tests.base.dummy import DummyContent
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.dummy import DummyUser
from Products.CMFCore.tests.base.testcase import SecurityTest
from Products.CMFCore.tests.base.tidata import FTIDATA_DUMMY
from Products.CMFCore.TypesTool import FactoryTypeInformation as FTI
from Products.CMFCore.TypesTool import TypesTool
from Products.CMFDefault.DiscussionTool import DiscussionTool
from Products.CMFDefault.exceptions import DiscussionNotAllowed

def has_path(catalog, path):
    if isinstance(path, tuple):
        path = '/'.join(path)
    return bool(catalog.getrid(path))


class DiscussionItemTests(unittest.TestCase):

    def test_interfaces(self):
        from Products.CMFCore.interfaces import ICatalogableDublinCore
        from Products.CMFCore.interfaces import IContentish
        from Products.CMFCore.interfaces import IDiscussionResponse
        from Products.CMFCore.interfaces import IDublinCore
        from Products.CMFCore.interfaces import IDynamicType
        from Products.CMFCore.interfaces import IMutableDublinCore
        from Products.CMFDefault.DiscussionItem import DiscussionItem

        verifyClass(ICatalogableDublinCore, DiscussionItem)
        verifyClass(IContentish, DiscussionItem)
        verifyClass(IDiscussionResponse, DiscussionItem)
        verifyClass(IDublinCore, DiscussionItem)
        verifyClass(IDynamicType, DiscussionItem)
        verifyClass(IMutableDublinCore, DiscussionItem)


class DiscussionItemContainerTests(unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFDefault.DiscussionItem import DiscussionItemContainer

        return DiscussionItemContainer

    def test_interfaces(self):
        from Products.CMFCore.interfaces import ICallableOpaqueItemEvents
        from Products.CMFCore.interfaces import IDiscussable

        verifyClass(ICallableOpaqueItemEvents, self._getTargetClass())
        verifyClass(IDiscussable, self._getTargetClass())


class DiscussionTests(SecurityTest):

    layer = EventZCMLLayer

    def setUp(self):
        SecurityTest.setUp(self)
        newSecurityManager(None, DummyUser().__of__(self.app.acl_users))
        self.site = DummySite('site').__of__(self.app)
        self.dtool = DiscussionTool()
        self.ttool = TypesTool()
        sm = getSiteManager()
        sm.registerUtility(self.dtool, IDiscussionTool)
        sm.registerUtility(self.ttool, ITypesTool)

    def _makeDummyContent(self, id, *args, **kw):
        return self.site._setObject(id, DummyContent(id, *args, **kw))

    def test_policy( self ):
        dtool = self.dtool
        ttool = self.ttool
        test = self._makeDummyContent('test')
        self.assertRaises(DiscussionNotAllowed, dtool.getDiscussionFor, test)
        assert getattr( test, 'talkback', None ) is None

        test.allow_discussion = 1
        assert dtool.getDiscussionFor(test)
        assert test.talkback

        del test.talkback
        del test.allow_discussion
        fti = FTIDATA_DUMMY[0].copy()
        ttool._setObject( 'Dummy Content', FTI(**fti) )
        self.assertRaises(DiscussionNotAllowed, dtool.getDiscussionFor, test)
        assert getattr( test, 'talkback', None ) is None

        ti = getattr(ttool, 'Dummy Content')
        ti.allow_discussion = 1
        assert dtool.getDiscussionFor(test)
        assert test.talkback

        del test.talkback
        ti.allow_discussion = 0
        self.assertRaises(DiscussionNotAllowed, dtool.getDiscussionFor, test)
        assert getattr( test, 'talkback', None ) is None

        test.allow_discussion = 1
        assert dtool.getDiscussionFor(test)
        assert test.talkback

    def test_nestedReplies( self ):
        dtool = self.dtool
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        talkback = dtool.getDiscussionFor(test)
        assert talkback._getDiscussable() == test
        assert talkback._getDiscussable( outer=1 ) == test
        assert not talkback.hasReplies( test )
        assert len( talkback.getReplies() ) == 0

        reply_id = talkback.createReply( title='test', text='blah' )
        assert talkback.hasReplies( test )
        assert len( talkback.getReplies() ) == 1
        assert talkback.getReply( reply_id )

        reply1 = talkback.getReplies()[0]
        items = talkback._container.items()
        self.assertEqual( reply1.getId(), items[0][0] )
        self.assertEqual( reply1.inReplyTo(), test )
        self.assertEqual( reply1.listCreators(), ('dummy',) )

        parents = reply1.parentsInThread()
        assert len( parents ) == 1
        assert test in parents

        talkback1 = dtool.getDiscussionFor(reply1)
        assert talkback == talkback1
        assert len( talkback1.getReplies() ) == 0
        assert len( talkback.getReplies() ) == 1

        talkback1.createReply( title='test2'
                             , text='blah2'
                             )
        assert len( talkback._container ) == 2
        assert talkback1.hasReplies( reply1 )
        assert len( talkback1.getReplies() ) == 1
        assert len( talkback.getReplies() ) == 1

        reply2 = talkback1.getReplies()[0]
        assert reply2.inReplyTo() == reply1

        parents = reply2.parentsInThread()
        assert len( parents ) == 2
        assert parents[ 0 ] == test
        assert parents[ 1 ] == reply1

        parents = reply2.parentsInThread( 1 )
        assert len( parents ) == 1
        assert parents[ 0 ] == reply1

    def test_itemCataloguing(self):
        ctool = CatalogTool()
        ctool.addColumn('in_reply_to')
        getSiteManager().registerUtility(ctool, ICatalogTool)
        dtool = self.dtool
        test = self._makeDummyContent('test', catalog=1)
        test.allow_discussion = 1

        self.assertEqual( len(ctool), 1 )
        self.failUnless(has_path(ctool, test.getPhysicalPath()))
        talkback = dtool.getDiscussionFor(test)
        self.assertEqual( talkback.getPhysicalPath(),
                          ('', 'bar', 'site', 'test', 'talkback') )
        talkback.createReply( title='test'
                            , text='blah'
                            )
        self.assertEqual( len(ctool), 2 )
        for reply in talkback.getReplies():
            self.failUnless(has_path(ctool, reply.getPhysicalPath()))
            self.failUnless(has_path(ctool, '/bar/site/test/talkback/%s'
                                            % reply.getId()))

        reply1 = talkback.getReplies()[0]
        path1 = '/'.join(reply1.getPhysicalPath())
        self.assertEqual( ctool.getMetadataForUID(path1),
                          {'in_reply_to': None} )

        talkback1 = dtool.getDiscussionFor(reply1)
        talkback1.createReply( title='test2'
                             , text='blah2'
                             )
        for reply in talkback.getReplies():
            self.failUnless(has_path(ctool, reply.getPhysicalPath()))
            self.failUnless(has_path(ctool, '/bar/site/test/talkback/%s'
                                            % reply.getId()))
        for reply in talkback1.getReplies():
            self.failUnless(has_path(ctool, reply.getPhysicalPath()))
            self.failUnless(has_path(ctool, '/bar/site/test/talkback/%s'
                                            % reply.getId()))

        reply2 = talkback1.getReplies()[0]
        path2 = '/'.join(reply2.getPhysicalPath())
        self.assertEqual( ctool.getMetadataForUID(path2),
                          {'in_reply_to': reply1.getId()} )

    def test_itemWorkflowNotification(self):
        from Products.CMFDefault.DiscussionItem import DiscussionItem

        dtool = self.dtool
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        talkback = dtool.getDiscussionFor(test)

        # Monkey patch into the class to test, urgh.
        def notifyWorkflowCreated(self):
            self.test_wf_notified = 1
            DiscussionItem.inheritedAttribute('notifyWorkflowCreated')(self)
        old_method = getattr(DiscussionItem, 'notifyWorkflowCreated', None)
        DiscussionItem.notifyWorkflowCreated = notifyWorkflowCreated
        DiscussionItem.test_wf_notified = 0

        try:
            talkback.createReply(title='test', text='blah')
            reply = talkback.getReplies()[0]
            self.assertEqual(reply.test_wf_notified, 1)
        finally:
            delattr(DiscussionItem, 'test_wf_notified')
            if old_method is None:
                delattr(DiscussionItem, 'notifyWorkflowCreated')
            else:
                DiscussionItem.notifyWorkflowCreated = old_method

    def test_deletePropagation( self ):
        ctool = CatalogTool()
        getSiteManager().registerUtility(ctool, ICatalogTool)
        dtool = self.dtool
        test = self._makeDummyContent('test', catalog=1)
        test.allow_discussion = 1

        talkback = dtool.getDiscussionFor(test)
        talkback.createReply( title='test'
                            , text='blah'
                            )
        self.assertEqual( len(ctool), 2 )
        self.site._delObject('test')
        self.assertEqual( len(ctool), 0 )

    def test_deleteReplies(self):
        dtool = self.dtool
        ctool = CatalogTool()
        getSiteManager().registerUtility(ctool, ICatalogTool)
        test = self._makeDummyContent('test')
        test.allow_discussion = 1

        # Create a structure 6 levels deep for testing
        talkback = dtool.getDiscussionFor(test)
        id1 = talkback.createReply(title='test1', text='blah')
        reply1 = talkback.getReply(id1)
        talkback1 = dtool.getDiscussionFor(reply1)
        id2 = talkback1.createReply(title='test2', text='blah')
        reply2 = talkback1.getReply(id2)
        talkback2 = dtool.getDiscussionFor(reply2)
        id3 = talkback2.createReply(title='test3', text='blah')
        reply3 = talkback2.getReply(id3)
        talkback3 = dtool.getDiscussionFor(reply3)
        id4 = talkback3.createReply(title='test4', text='blah')
        reply4 = talkback3.getReply(id4)
        talkback4 = dtool.getDiscussionFor(reply4)
        id5 = talkback4.createReply(title='test5', text='blah')
        reply5 = talkback4.getReply(id5)
        talkback5 = dtool.getDiscussionFor(reply5)
        id6 = talkback5.createReply(title='test6', text='blah')
        reply6 = talkback5.getReply(id6)
        talkback6 = dtool.getDiscussionFor(reply6)

        self.assertEqual(len(talkback.getReplies()), 1)
        self.assertEqual(len(talkback1.getReplies()), 1)
        self.assertEqual(len(talkback2.getReplies()), 1)
        self.assertEqual(len(talkback3.getReplies()), 1)
        self.assertEqual(len(talkback4.getReplies()), 1)
        self.assertEqual(len(talkback5.getReplies()), 1)
        self.assertEqual(len(talkback6.getReplies()), 0)
        self.assertEqual(len(ctool), 7)

        talkback3.deleteReply(id4)
        self.assertEqual(len(talkback.getReplies()), 1)
        self.assertEqual(len(talkback1.getReplies()), 1)
        self.assertEqual(len(talkback2.getReplies()), 1)
        self.assertEqual(len(talkback3.getReplies()), 0)
        self.assertEqual(len(ctool), 4)

        talkback.deleteReply(id1)
        self.assertEqual(len(talkback.getReplies()), 0)
        self.assertEqual(len(ctool), 1)

    def test_newTalkbackIsWrapped(self):
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        dtool = self.dtool
        talkback = dtool.getDiscussionFor(test)
        self.failUnless(hasattr(talkback, 'aq_base'))
        # Acquire REQUEST
        self.failUnless(getattr(talkback, 'REQUEST'))

    def test_existingTalkbackIsWrapped(self):
        test = self._makeDummyContent('test')
        test.allow_discussion = 1
        dtool = self.dtool
        dtool.getDiscussionFor(test)
        talkback = dtool.getDiscussionFor(test)
        self.failUnless(hasattr(talkback, 'aq_base'))
        # Acquire REQUEST
        self.failUnless(getattr(talkback, 'REQUEST'))


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(DiscussionItemTests),
        unittest.makeSuite(DiscussionItemContainerTests),
        unittest.makeSuite(DiscussionTests),
        ))
